// Snow Datepicker v1.1.6
// https://github.com/janghye0k/snow-datepicker
// (c) 2023 - 2024 JangHyeok Kim
// Snow Datepicker may be freely distributed under the MIT license.
const FORMAT_REGEXP = /\[([^\]]+)]|YY[1|2]|YYYY[1|2]|Y{2,4}|M{1,4}|D{1,2}|d{1,4}|H{1,2}|h{1,2}|a|A|m{1,2}|s{1,2}|Z{1,2}|SSS/g;
const DATEFORMAT_REGEXP = /\[([^\]]+)]|Y{2,4}|M{1,4}|D{1,2}/g;
const DEFAULT_FORMAT = "YYYY-MM-DDTHH:mm:ssZ";
const VIEW_ORDER = {
  days: 1,
  months: 2,
  years: 3
};
const VIEW_LIST = ["days", "months", "years"];
const MIN_DATE = new Date(100, 0, 1, 0, 1, 0);
const MAX_DATE = new Date(9999, 11, 31, 23, 59, 59);
const PREFIX = "snow-datepicker";
const CONTAINER_ID = `${PREFIX}-container`;
const BUTTON_PRESETS = ["clear", "today"];
const DEFUALT_OPTIONS = {
  toggleSelected: true,
  shortcuts: true,
  position: "bottom-start",
  view: "days",
  minView: "days",
  showOtherMonths: true,
  selectOtherMonths: true,
  moveOtherMonths: true,
  navigationLoop: true,
  autoClose: true,
  readOnly: false,
  animation: true,
  inline: false,
  backdrop: false,
  theme: "light",
  size: "normal",
  titleFormat: {}
};
const DEFAULT_TITLE_FORMAT = {
  days: "MMMM, [<i>]YYYY[</i>]",
  months: "YYYY",
  years: "YYYY1 - YYYY2"
};
const OPTION_EVENT_KEYS = [
  "onShow",
  "onHide",
  "onFocus",
  "onBlur",
  "onClickCell",
  "onRenderCell",
  "onBeforeSelect",
  "onSelect",
  "onChangeView",
  "onChangeViewDate"
];

const classNameMap = {
  controls: "controls",
  controlsBtn: "controls__btn",
  controlsTitle: "controls__title",
  content: "content",
  days: "content__days",
  months: "content__months",
  years: "content__years",
  weekday: "content__weekday",
  cell: "content__cell",
  calendar: "calendar",
  inputRoot: "inputRoot",
  inputBox: "inputBox",
  label: "inputLabel",
  input: "input",
  backdrop: "backdrop",
  calendarBtn: "calendarBtn",
  button: "button",
  buttonWrapper: "buttonWrapper"
};
const cn = (key) => PREFIX + "-" + classNameMap[key];

// DOUMI v1.1.7
// https://github.com/janghye0k/doumi
// (c) 2024 - 2024 JangHyeok Kim
// Doumi may be freely distributed under the MIT license.

const tagType = (value) => Object.prototype.toString.call(value).slice(`[object `.length, -1);

const isArray = (value) => Array.isArray(value);

const isArrayLike = (value) => {
  if (!value || typeof value === "function" || typeof (value == null ? void 0 : value.length) !== "number")
    return false;
  return value.length > -1;
};

const isNull = (value) => value === null;

const isUndefined = (value) => value === void 0;

const isNullish = (value) => isNull(value) || isUndefined(value);

const isString = (value) => typeof value === "string" || tagType(value) === "String";

const isBoolean = (value) => typeof value === "boolean" || tagType(value) === "Boolean";

const isDate = (value) => tagType(value) === "Date" && !isNaN(value);

const isDateLike = (value) => {
  if (!value)
    return false;
  return !isNaN(new Date(value));
};

const isObject = (value) => {
  return typeof value === "function" || typeof value === "object" && !!value;
};

const isPlainObject = (value) => {
  return !!value && typeof value === "object" && tagType(value) === "Object";
};

const isFunction = (value) => typeof value === "function";

const isNumber = (value) => typeof value === "number" || tagType(value) === "Number";

const isSymbol = (value) => typeof value === "symbol" || tagType(value) === "Symbol";

const assign = Object.assign;

const keysIn = (object) => {
  const keys = [];
  for (const key in object) {
    if (!isUndefined(key))
      keys.push(key);
  }
  return keys;
};

function assignIn(object, ...sources) {
  sources.forEach((source) => {
    if (!isObject(source))
      return;
    keysIn(source).forEach((key) => {
      object[key] = source[key];
    });
  });
  return object;
}

function isKey(value) {
  if (isArray(value))
    return false;
  if (isNumber(value) || isSymbol(value) || isBoolean(value))
    return true;
  return /^\w*$/.test(value);
}
const bracketReg = new RegExp(
  String.raw`[^.[\]]+|\[(?:\-\d+|\d+|\".*\"|\'.*\')\]`,
  "gi"
);
function splitObjectPath(paths) {
  if (isKey(paths))
    return [paths];
  const results = [];
  let str = paths;
  while (str.at(0) === ".") {
    results.push("");
    str = str.slice(1);
  }
  str.replace(bracketReg, (match) => {
    if (match.startsWith('["') || match.startsWith("['")) {
      results.push(match.slice(2, -2));
    } else if (match.startsWith("[")) {
      results.push(match.slice(1, -1));
    } else
      results.push(match);
    return "";
  });
  return results;
}
const numRegex = /^(\d+|-\d+)$/;
const isIndex = (value) => numRegex.test(value) && Number.isInteger(Number(value));

const get = (object, paths) => {
  if (!isObject(object))
    return void 0;
  let pointer = object;
  const splited = splitObjectPath(paths);
  for (const item of splited) {
    if (!isObject(pointer))
      return void 0;
    if (isArray(pointer) && isIndex(item))
      pointer = pointer.at(Number(item));
    else
      pointer = pointer[item];
  }
  return pointer;
};

const keys = (object) => {
  return Object.keys(object);
};

const values = (object) => {
  return Object.values(object);
};

const entries = (object) => {
  return keys(object).map((key) => {
    const value = object[key];
    return [value, key];
  });
};

function forEach(collection, callback) {
  if (isArrayLike(collection)) {
    for (let i = 0; i < collection.length; i++) {
      callback(collection[i], i, collection);
    }
  } else if (isObject(collection)) {
    const keyList = keys(collection);
    for (const key of keyList) {
      callback(collection[key], key, collection);
    }
  }
}

function reduce(collection, reducer, accumulator) {
  let results = accumulator;
  if (isArrayLike(collection)) {
    if (isUndefined(results))
      results = collection[0];
    for (let i = 0; i < collection.length; i++) {
      results = reducer(results, collection[i], i, collection);
    }
  } else if (isObject(collection)) {
    const keyList = keys(collection);
    if (isUndefined(results))
      results = collection[keyList[0]];
    for (const key of keyList) {
      results = reducer(results, collection[key], key, collection);
    }
  }
  return results;
}

function range(...args) {
  const arr = [];
  let std, end;
  if (args.length !== 1) {
    std = args[0];
    end = std + args[1];
  } else
    std = 0, end = args[0];
  for (let i = std; i < end; i++)
    arr.push(i);
  return arr;
}

function find$(selector, element = document) {
  return element.querySelector(selector);
}

function findAll$(selector, element = document) {
  return element.querySelectorAll(selector);
}

function on(element, eventType, listener, options) {
  element.addEventListener(eventType, listener, options);
}

function off(element, eventType, listener) {
  element.removeEventListener(eventType, listener);
}

function create$(tagName, options = {}) {
  const $el = document.createElement(tagName);
  const { id, className, classList, role, style, dataset, ...props } = options;
  if (id)
    $el.id = id;
  if (className)
    $el.classList.add(...className.trim().split(" "));
  if (Array.isArray(classList))
    $el.classList.add(...classList);
  if (role)
    $el.role = role;
  if (style && tagType(style) === "Object") {
    for (const key in style) {
      const value = style[key];
      if (value === void 0)
        continue;
      $el.style[key] = value;
    }
  }
  if (tagType(dataset) === "Object") {
    for (const key in dataset) {
      const value = dataset[key];
      if (value === void 0)
        continue;
      $el.dataset[key] = `${value}`;
    }
  }
  if (tagType(props) === "Object") {
    for (const key in props) {
      const value = props[key];
      if (value === void 0)
        continue;
      $el[key] = value;
    }
  }
  return $el;
}

function between(number, start, end) {
  return number >= start && number <= end;
}

const capitalize = (str) => str.charAt(0).toUpperCase() + str.slice(1).toLowerCase();

class Components {
  constructor(element, { dp, instance, options, eventManager, ...props }) {
    this.unsubscribers = [];
    this.$el = element;
    this.dp = dp;
    this.instance = instance;
    this.options = options;
    this.eventManager = eventManager;
    this.assignProps(props);
    this.beforeInit();
    this.init();
  }
  beforeInit() {
  }
  init() {
    this.render();
    this.bindEvents();
    this.subscribe();
  }
  bindEvents() {
  }
  render() {
  }
  subscribe() {
  }
  beforeDestroy() {
  }
  assignProps(props) {
    assign(this, props);
  }
  destroy() {
    this.beforeDestroy();
    this.unsubscribers.forEach((fn) => fn());
    this.$el.remove();
  }
}

class Button extends Components {
  render() {
    let $el;
    if (isObject(this.buttonOption)) {
      const { id, className, innerHTML, style, dataset, attrs, onClick } = this.buttonOption;
      const classList = [cn("button")];
      if (className) classList.push(className);
      $el = create$("button", { id, classList, innerHTML, dataset, style });
      if (attrs) {
        forEach(attrs, (value, key) => $el.setAttribute(key, value));
      }
      if (onClick) this.onClick = (e) => onClick(e, this.dp);
    } else {
      this.onClick = this.buttonOption === "clear" ? () => this.dp.setSelectedDate(null) : () => this.dp.setSelectedDate(/* @__PURE__ */ new Date());
      $el = create$("button", {
        className: cn("button"),
        innerHTML: capitalize(this.buttonOption)
      });
    }
    const $origin = this.$el;
    this.$el = $el;
    $origin.replaceWith($el);
  }
  bindEvents() {
    on(this.$el, "click", this.onClick);
  }
}

function error(title, body) {
  const messages = [`[SnowDatePicker]: ${title}`];
  if (typeof body === "string") messages.push(body);
  throw new Error(messages.join("\n"));
}
const parseDate = (date = /* @__PURE__ */ new Date()) => {
  if (!isDate(date)) return [-1, -1, -1];
  return [date.getFullYear(), date.getMonth(), date.getDate()];
};
const decade = (date) => {
  const currentUnitYear = date.getFullYear();
  const yearPerDecade = Math.floor(currentUnitYear / 10);
  const start = yearPerDecade * 10;
  const end = start + 9;
  return [start, end];
};
const getCalendarDates = (date, weekStart = 0) => {
  const [year, month] = parseDate(date);
  const lastDate = new Date(year, month + 1, 0);
  const firstDayIndex = new Date(year, month, 1).getDay();
  const lastDayIndex = lastDate.getDay();
  const preDaysCount = (firstDayIndex + 7 - weekStart) % 7 || (lastDate.getDate() === 28 ? 7 : 0);
  const postDaysCount = 6 - lastDayIndex;
  const totalDaysCount = preDaysCount + postDaysCount + lastDate.getDate();
  const results = range(-preDaysCount + 1, totalDaysCount).map(
    (dayindex) => new Date(year, month, dayindex, 12)
  );
  return results;
};

function _iterableToArrayLimit(r, l) {
  var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"];
  if (null != t) {
    var e,
      n,
      i,
      u,
      a = [],
      f = !0,
      o = !1;
    try {
      if (i = (t = t.call(r)).next, 0 === l) {
        if (Object(t) !== t) return;
        f = !1;
      } else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0);
    } catch (r) {
      o = !0, n = r;
    } finally {
      try {
        if (!f && null != t.return && (u = t.return(), Object(u) !== u)) return;
      } finally {
        if (o) throw n;
      }
    }
    return a;
  }
}
function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}
function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}
function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

/**
 * Check parameters are equal
 * @param  {*} value1
 * @param  {*} value2
 * @return {boolean} Returns true if they're equal in value
 */
function isEqual(value1, value2) {
  /**
   * check type
   * @param  {any} val
   * @return {string}
   */
  function typeOf(val) {
    return Object.prototype.toString.call(val).slice(8, -1).toLowerCase();
  }
  // Get the value type
  var type = typeOf(value1);
  if (type !== typeOf(value2)) return false;
  if (type === 'array') {
    if (value1.length !== value2.length) return false;
    for (var i = 0; i < value1.length; i++) {
      if (!isEqual(value1[i], value2[i])) return false;
    }
    return true;
  }
  if (type === 'object') {
    if (Object.keys(value1).length !== Object.keys(value2).length) return false;
    for (var key in value1) {
      if (Object.prototype.hasOwnProperty.call(value1, key)) {
        if (!isEqual(value1[key], value2[key])) return false;
      }
    }
    return true;
  }
  if (type === 'function') return value1.toString() === value2.toString();
  return value1 === value2;
}

var defaultCompare = function defaultCompare(state) {
  return state;
};
function effect(callback, _ref) {
  var _ref2 = _slicedToArray(_ref, 2),
    observable = _ref2[0],
    immediate = _ref2[1];
  for (var _len = arguments.length, options = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    options[_key - 2] = arguments[_key];
  }
  var option1 = options[0],
    option2 = options[1];
  var compare = defaultCompare;
  var defaults = compare(observable());
  if (typeof option1 === 'function') {
    compare = option1;
    defaults = options.length > 1 ? option2 : compare(observable());
  } else if (options.length === 1) defaults = option1;
  var previousValue = immediate ? !defaults : defaults;
  return observable.subscribe(function (state, prevState) {
    var currentValue = compare(state);
    if (isEqual(currentValue, previousValue)) return;
    previousValue = currentValue;
    callback(state, prevState);
  }, immediate);
}

function observable() {
  // JS functions can't inherit custom prototypes, so we use prop() as a
  // proxy to the real ObservableValue instead.
  var observable = new ObservableValue();
  function prop() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    return observable.accessor.apply(prop, args);
  }
  var _loop = function _loop(key) {
    if (typeof observable[key] === 'function') {
      prop[key] = observable[key];
    } else {
      Object.defineProperty(prop, key, {
        get: function get() {
          return observable[key];
        },
        set: function set(value) {
          observable[key] = value;
        }
      });
    }
  };
  for (var key in observable) {
    _loop(key);
  }
  prop.apply(void 0, arguments);
  return prop;
}
function ObservableValue() {
  this.previousValue = null;
  this.value = null;
  this.listeners = [];
}
ObservableValue._computeActive = false;
ObservableValue._computeChildren = [];
ObservableValue.prototype.accessor = function accessor(newValue) {
  var _this = this;
  // If no arguments, return the value. If called inside a computed observable
  // value function, track child observables.
  if (!arguments.length) {
    if (ObservableValue._computeActive && ObservableValue._computeChildren.indexOf(this) === -1) {
      ObservableValue._computeChildren.push(this);
    }
    return this.value;
  }
  // If new value is same as previous, skip.
  else if (newValue !== this.value) {
    // If new value is not a function, save and publish.
    if (typeof newValue !== 'function') {
      this.previousValue = this.value;
      this.value = newValue;
      this.publish();
    }
    // If new value is a function, call the function and save its result.
    // Function can return a promise for async assignment. All additional
    // arguments are passed to the value function.
    else {
      var args = [];
      for (var i = 1; i < arguments.length; i++) {
        var arg = arguments[i];
        args.push(arg);
      }
      this.valueFunction = newValue;
      this.valueFunctionArgs = args;
      // Subscribe to child observables
      ObservableValue._computeActive = true;
      this.compute();
      ObservableValue._computeActive = false;
      ObservableValue._computeChildren.forEach(function (child) {
        child.subscribe(function () {
          return _this.compute();
        });
      });
      ObservableValue._computeChildren.length = 0;
    }
  }
};
ObservableValue.prototype.publish = function publish() {
  var _this2 = this;
  this.listeners.slice().forEach(function (listener) {
    if (!listener) return;
    listener.call(_this2, _this2.value, _this2.previousValue);
  });
};
ObservableValue.prototype.subscribe = function subscribe(listener, immediate) {
  var _this3 = this;
  this.listeners.push(listener);
  if (immediate) {
    listener.call(this, this.value, this.previousValue);
  }
  return function () {
    return ObservableValue.prototype.unsubscribe.call(_this3, listener);
  };
};
ObservableValue.prototype.unsubscribe = function unsubscribe(listener) {
  var index = this.listeners.indexOf(listener);
  this.listeners.splice(index, 1);
};
// Note, currently there's no shortcut to cleanup a computed value.
ObservableValue.prototype.compute = function compute() {
  var _this$valueFunction,
    _this4 = this;
  var result = (_this$valueFunction = this.valueFunction).call.apply(_this$valueFunction, [this].concat(_toConsumableArray(this.valueFunctionArgs)));
  if (typeof result !== 'undefined') {
    if (typeof result.then === 'function') {
      result.then(function (asyncResult) {
        return _this4(asyncResult);
      });
    } else {
      this(result);
    }
  }
};

const typeCompareLengthMap = {
  day: 3,
  month: 2,
  year: 1
};
class Cell extends Components {
  isActive() {
    const { selectedDate } = this.instance.store;
    return !isNull(selectedDate) && this.dp.isSameDate(this.date, selectedDate, this.type + "s");
  }
  isFocus() {
    const { focusDate } = this.dp;
    return !isNull(focusDate) && this.dp.isSameDate(this.date, focusDate, this.type + "s");
  }
  isToday() {
    return this.dp.isSameDate(this.date, /* @__PURE__ */ new Date(), this.type + "s");
  }
  render() {
    var _a;
    const { viewDate } = this.instance.store;
    const { minDate, maxDate } = this.options;
    const length = typeCompareLengthMap[this.type];
    const parsed = parseDate(this.date).slice(0, length);
    const [year, monthindex, day] = parsed;
    const viewMonth = viewDate.getMonth();
    const isActive = this.isActive();
    const classList = [cn("cell"), `--${this.type}`];
    if (this.isToday()) classList.push("--today");
    if (isActive) classList.push("--active");
    if (this.isFocus()) classList.push("--focus");
    if (this.type === "day") {
      const isOtherMonth = monthindex !== viewMonth;
      const isHidden = isOtherMonth && this.options.showOtherMonths === false;
      const isDisabled = isOtherMonth && this.options.selectOtherMonths === false;
      if (isOtherMonth) classList.push("--other");
      if (isHidden) classList.push("--hidden");
      if (isDisabled) classList.push("--disabled");
    }
    if (this.type === "year") {
      const [std, end] = decade(viewDate);
      if (!between(year, std, end)) classList.push("--other");
    }
    const N = Number;
    const isOver = !between(N(this.date), N(minDate), N(maxDate));
    if (isOver) classList.push("--disabled");
    const $el = create$("button", {
      classList,
      role: "gridcell",
      tabIndex: isActive ? 0 : -1,
      dataset: { year, monthindex, day },
      innerHTML: `${parsed[length - 1] + (this.type === "month" ? 1 : 0)}`
    });
    if (this.type === "day" && this.date.getDay() === 0)
      $el.classList.add("--dayoff");
    const eventProps = { date: this.date, type: this.type, $element: $el };
    const results = [];
    ((_a = this.eventManager.trigger("renderCell", eventProps)) != null ? _a : []).forEach(
      (result) => {
        if (isPlainObject(result)) results.push(result);
      }
    );
    results.forEach(({ className, innerHTML, dataset, attrs }) => {
      if (isString(className) && className) $el.classList.add(className);
      if (isString(innerHTML) && innerHTML) $el.innerHTML = innerHTML;
      if (isPlainObject(dataset))
        entries(dataset).forEach(
          ([value, key]) => $el.dataset[key] = `${value}`
        );
      if (isPlainObject(attrs)) {
        entries(attrs).forEach(
          ([value, key]) => $el.setAttribute(key, `${value}`)
        );
      }
    });
    const $origin = this.$el;
    this.$el = $el;
    $origin.replaceWith(this.$el);
    this.$el.dpCell = this;
  }
  bindEvents() {
    on(this.$el, "click", () => this.handleSelectCell());
    on(this.$el, "click", (evt) => this.handleClickCell(evt));
    on(this.$el, "mouseenter", () => this.handleMouseEnter());
  }
  handleMouseEnter() {
    this.instance.store.state.focusDate(this.date);
  }
  handleSelectCell() {
    const { currentView } = this.instance.store;
    const nextView = this.type === "year" ? "months" : "days";
    if (currentView !== nextView) this.dp.setCurrentView(nextView);
    const date = this.date;
    const isEndView = this.type + "s" === this.options.minView;
    if (isEndView) {
      const isUnSelect = this.isActive() && this.options.toggleSelected;
      this.dp.setSelectedDate(isUnSelect ? null : date);
      if (this.options.autoClose) this.dp.hide();
    }
    if (currentView !== "days" || this.options.moveOtherMonths) {
      this.dp.setViewDate(date);
    }
  }
  handleClickCell(event) {
    const { type, date, $el: $element } = this;
    const eventProps = { event, type, date, $element };
    this.eventManager.trigger("clickCell", eventProps);
  }
}

class Content extends Components {
  constructor() {
    super(...arguments);
    this.cells = [];
  }
  subscribe() {
    const { store } = this.instance;
    this.unsubscribers.push(
      effect(
        (state) => {
          this.$el.className = [cn("content"), `--${state}`].join(" ");
        },
        [store.state.currentView, true]
      ),
      effect(() => {
        this.renderCells();
      }, [store.state.viewState, true]),
      effect(() => {
        const $cells = findAll$("." + cn("cell"), this.$el);
        $cells.forEach(($cell) => {
          const cell = $cell.dpCell;
          if (cell.isActive()) {
            $cell.classList.add("--active");
            $cell.tabIndex = 0;
          } else {
            $cell.classList.remove("--active");
            $cell.tabIndex = -1;
          }
        });
      }, [store.state.date, true]),
      effect(
        (focusDate) => {
          const [year, monthindex, day] = parseDate(focusDate);
          const currentView = store.currentView;
          const comapre = (date) => {
            let isSame = true;
            isSame = isSame && date.getFullYear() === year;
            if (currentView === "years") return isSame;
            isSame = isSame && date.getMonth() === monthindex;
            if (currentView === "months") return isSame;
            return isSame && date.getDate() === day;
          };
          const $cells = findAll$("." + cn("cell"), this.$el);
          $cells.forEach(($cell) => {
            $cell.classList.remove("--focus");
            const { date, type } = $cell.dpCell;
            const isFocus = comapre(date);
            if (isFocus) {
              $cell.focus(), $cell.classList.add("--focus");
              const eventProps = {
                $element: $cell,
                type,
                date
              };
              this.eventManager.trigger("focus", eventProps);
            }
          });
        },
        [store.state.focusDate]
      )
    );
  }
  bindEvents() {
    on(this.$el, "mouseleave", () => this.instance.store.state.focusDate(null));
  }
  generateWeekDays() {
    const {
      locale: { weekdaysMin },
      weekIndexes
    } = this.instance.converter;
    return weekIndexes.map((index) => {
      const $weekday = create$("div", {
        role: "gridcell",
        className: cn("weekday"),
        dataset: { dayindex: index },
        innerHTML: weekdaysMin[index]
      });
      return $weekday;
    });
  }
  generateDates() {
    const { store, converter } = this.instance;
    const { viewDate, currentView } = store;
    switch (currentView) {
      case "days":
        return getCalendarDates(viewDate, converter.locale.weekStart);
      case "months":
        const year = viewDate.getFullYear();
        return range(12).map((idx) => new Date(year, idx, 1, 12));
      case "years":
        return range(decade(viewDate)[0] - 1, 12).map(
          (year2) => new Date(year2, 0, 1, 12)
        );
    }
  }
  render() {
    this.$el.role = "grid";
  }
  renderCells() {
    const { instance, options, dp, eventManager } = this;
    const type = instance.store.currentView.slice(0, -1);
    const dates = this.generateDates();
    if (!isArray(this.cells)) this.cells = [];
    this.cells.forEach((cell) => cell.destroy());
    this.$el.innerHTML = "";
    if (type === "day") {
      const $weekdays = this.generateWeekDays();
      this.$el.replaceChildren(...$weekdays);
    }
    dates.forEach((date) => {
      const $cell = create$("button");
      this.$el.appendChild($cell);
      new Cell($cell, { type, date, dp, instance, options, eventManager });
    });
  }
}

var ChevronLeft = `<svg viewBox="0 0 24 24" width="18" height="18" fill="none" xmlns="http://www.w3.org/2000/svg" transform="matrix(-1, 0, 0, 1, 0, 0)"><g id="SVGRepo_bgCarrier" stroke-width="0"></g><g id="SVGRepo_tracerCarrier" stroke-linecap="round" stroke-linejoin="round"></g><g id="SVGRepo_iconCarrier"> <path d="M9 6L15 12L9 18" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"></path> </g></svg>`;

var ChevronRight = `<svg viewBox="0 0 24 24" width="18" height="18" fill="none" xmlns="http://www.w3.org/2000/svg"><g id="SVGRepo_bgCarrier" stroke-width="0"></g><g id="SVGRepo_tracerCarrier" stroke-linecap="round" stroke-linejoin="round"></g><g id="SVGRepo_iconCarrier"> <path d="M9 6L15 12L9 18" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"></path> </g></svg>`;

class Controls extends Components {
  render() {
    this.$el.role = "navigation";
    this.$prevBtn = create$("button", {
      className: cn("controlsBtn"),
      dataset: { action: "prev" },
      innerHTML: ChevronLeft
    });
    this.$nextBtn = create$("button", {
      className: cn("controlsBtn"),
      dataset: { action: "next" },
      innerHTML: ChevronRight
    });
    this.$title = create$("span", { className: cn("controlsTitle") });
    this.$el.innerHTML = "";
    this.$el.replaceChildren(this.$prevBtn, this.$title, this.$nextBtn);
  }
  bindEvents() {
    const { store } = this.instance;
    on(this.$title, "click", () => {
      const nextView = store.currentView === "days" ? "months" : "years";
      store.setCurrentView(nextView);
    });
    on(this.$prevBtn, "click", (e) => {
      e.preventDefault();
      this.dp.prev(), this.dp.setFocusDate(null);
    });
    on(this.$nextBtn, "click", (e) => {
      e.preventDefault();
      this.dp.next(), this.dp.setFocusDate(null);
    });
    [this.$prevBtn, this.$nextBtn].forEach(($el) => {
      on($el, "keydown", (evt) => evt.key === "Enter" && $el.click());
      on($el, "focus", () => this.instance.store.state.focusDate(null));
    });
  }
  subscribe() {
    const { store, converter } = this.instance;
    const { titleFormat, minDate, maxDate } = this.options;
    const parsed = { min: parseDate(minDate), max: parseDate(maxDate) };
    const decades = { min: decade(minDate), max: decade(maxDate) };
    const btnHideCheckMap = {
      days: (year, monthindex) => {
        if (year <= parsed.min[0] && monthindex - 1 < parsed.min[1]) {
          this.$prevBtn.disabled = true;
        } else if (year >= parsed.max[0] && monthindex + 1 > parsed.max[1]) {
          this.$nextBtn.disabled = true;
        }
      },
      months: (year) => {
        if (year <= parsed.min[0]) {
          this.$prevBtn.disabled = true;
        } else if (year >= parsed.max[0]) {
          this.$nextBtn.disabled = true;
        }
      },
      years: (year) => {
        if (year <= decades.min[0]) {
          this.$prevBtn.disabled = true;
        } else if (year >= decades.max[1]) {
          this.$nextBtn.disabled = true;
        }
      }
    };
    this.unsubscribers.push(
      effect(
        (state) => {
          const [currentView, year, monthindex] = state.split("__");
          const format = titleFormat[currentView];
          this.$title.innerHTML = converter.format(this.dp.viewDate, format);
          if (!this.options.navigationLoop) {
            this.$prevBtn.disabled = false;
            this.$nextBtn.disabled = false;
            btnHideCheckMap[currentView](
              Number(year),
              Number(monthindex)
            );
          }
        },
        [store.state.viewState, true]
      )
    );
  }
}

var CalendarIcon = `
<svg version="1.1" id="mdi-calendar-range" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px"
	 y="0px" viewBox="0 0 512 512" style="enable-background:new 0 0 512 512;" xml:space="preserve">
<g>
	<path d="M179.2,230.4H128v51.2h51.2V230.4 M281.6,230.4h-51.2v51.2h51.2V230.4 M384,230.4h-51.2v51.2H384V230.4 M435.2,51.2h-25.6
		V0h-51.2v51.2H153.6V0h-51.2v51.2H76.8c-28.42,0-51.2,23.04-51.2,51.2v358.4c0,28.28,22.92,51.2,51.2,51.2h358.4
		c28.28,0,51.2-22.92,51.2-51.2V102.4C486.4,74.12,463.48,51.2,435.2,51.2 M435.2,460.8H76.8V179.2h358.4V460.8z"/>
</g>
</svg>
`;

const IGNORE_REGEXP = /[^Y|M|D|\d]*/g;
function createInputState(dateFormat, minView) {
  const beforeStrMap = {
    year: "",
    month: "",
    day: "",
    rest: ""
  };
  const formatStrMap = {
    year: "",
    month: "",
    day: ""
  };
  let keyOrders = [];
  const valueMap = {};
  let prevIdx = 0;
  const rest = dateFormat.replace(
    DATEFORMAT_REGEXP,
    (match, escapeStr, idx) => {
      if (escapeStr) {
        return `[${escapeStr}]`;
      }
      const length = match.length;
      const prevStr = dateFormat.slice(prevIdx, idx);
      prevIdx = idx + length;
      const has = (val) => match.includes(val);
      const target = has("Y") ? "year" : has("M") ? "month" : "day";
      formatStrMap[target] = match;
      beforeStrMap[target] = prevStr;
      keyOrders.push(target);
      return "".padStart(length, " ");
    }
  );
  beforeStrMap.rest = rest.slice(prevIdx);
  const minOrder = VIEW_ORDER[minView];
  keyOrders = keyOrders.filter((key) => VIEW_ORDER[key + "s"] >= minOrder);
  return {
    beforeStrMap,
    formatStrMap,
    keyOrders,
    valueMap,
    format: dateFormat
  };
}
class Target extends Components {
  get inputDate() {
    var _a;
    const { year, month, day } = this.inputState.valueMap;
    let y = year;
    if (this.inputState.formatStrMap.year.length === 2) {
      y = Number(
        `${(_a = (/* @__PURE__ */ new Date()).getFullYear()) == null ? void 0 : _a.toString().slice(0, 2)}${year == null ? void 0 : year.toString().padStart(2, "0")}`
      );
    }
    return new Date(y, month - 1, day);
  }
  get displayFormat() {
    const { keyOrders, beforeStrMap, formatStrMap } = this.inputState;
    return keyOrders.reduce((acc, key) => {
      return acc + beforeStrMap[key] + formatStrMap[key];
    }, "");
  }
  beforeDestroy() {
    this.$el = create$("input");
  }
  subscribe() {
    const { store, converter } = this.instance;
    this.unsubscribers.push(
      effect(
        (date) => {
          if (!date)
            return this.$input.value = converter.format(
              date,
              this.displayFormat
            );
          const [year, monthindex, day] = parseDate(date);
          const { valueMap } = this.inputState;
          if (this.inputState.formatStrMap.year.length === 2) {
            valueMap.year = Number(year.toString().slice(-2));
          } else {
            valueMap.year = year;
          }
          valueMap.day = day;
          valueMap.month = monthindex + 1;
          this.$input.value = converter.format(date, this.displayFormat);
        },
        [store.state.date]
      )
    );
  }
  beforeInit() {
    var _a;
    const inputState = createInputState(
      (_a = this.options.dateFormat) != null ? _a : this.instance.converter.locale.formats.date,
      this.options.minView
    );
    assignIn(this, { inputState });
  }
  render() {
    var _a, _b;
    this.$el.classList.add(PREFIX, cn("inputRoot"));
    const $inputBox = create$("div", { className: cn("inputBox") });
    const $input = create$("input", {
      className: cn("input"),
      type: "text",
      placeholder: (_b = (_a = this.options.placeHolder) != null ? _a : this.instance.converter.locale.placeholder) != null ? _b : "Choose a date",
      readOnly: this.options.readOnly,
      value: this.instance.converter.format(null, this.displayFormat)
    });
    assignIn(this, { $input, $inputBox });
    const $iconBox = create$("button", {
      className: cn("calendarBtn"),
      innerHTML: CalendarIcon
    });
    this.$inputBox.replaceChildren($input, $iconBox);
    this.$el.appendChild(this.$inputBox);
    assignIn(this.dp, { $input });
  }
  bindEvents() {
    on(
      find$("." + cn("calendarBtn"), this.dp.$target),
      "click",
      () => this.handleClickCalendarButton()
    );
    on(this.$input, "drop", (event) => {
      event.dataTransfer.dropEffect = "none";
      event.stopPropagation();
      event.preventDefault();
    });
    on(
      this.$input,
      "pointerdown",
      (evt) => assignIn(evt.currentTarget, { isPointerDown: true })
    );
    on(this.$input, "blur", (evt) => this.handleBlur(evt));
    on(this.$input, "focus", (evt) => this.handleFocus(evt));
    on(this.$input, "pointerup", (evt) => this.handlePointerUp(evt));
    on(
      document,
      "keydown",
      (e) => {
        var _a;
        if (!((_a = document.activeElement) == null ? void 0 : _a.isSameNode(this.$input))) return;
        this.handleKeydown(e);
        this.handleKeydownInput(e);
      },
      true
    );
    on(this.$input, "paste", (evt) => this.handlePaste(evt));
  }
  handleClickCalendarButton() {
    this.dp.isShow() ? this.dp.hide() : this.dp.show();
  }
  formatTargetValue(key) {
    const { valueMap, formatStrMap } = this.inputState;
    const targetValue = valueMap[key];
    const targetFormat = formatStrMap[key];
    if (!targetValue) return targetFormat;
    if (key !== "month") {
      return String(targetValue).padStart(targetFormat.length, "0");
    }
    const formatMonthSize = targetFormat.length;
    if (formatMonthSize <= 2) {
      return String(targetValue).padStart(formatMonthSize, "0");
    }
    const { converter } = this.instance;
    return converter.localeMonth(targetValue - 1, formatMonthSize === 3);
  }
  getIndexMap() {
    const { beforeStrMap, keyOrders } = this.inputState;
    const indexMap = {};
    let idx = 0;
    keyOrders.forEach((key) => {
      const beforeStr = beforeStrMap[key].replace(
        /\[([^\]]+)]/g,
        (_, escapeStr) => escapeStr
      );
      idx += beforeStr.length;
      const valueSize = this.formatTargetValue(key).length;
      indexMap[key] = [idx, idx += valueSize];
    });
    return indexMap;
  }
  convertInputStateValue() {
    const { keyOrders, beforeStrMap } = this.inputState;
    return keyOrders.reduce((acc, key) => {
      return acc + beforeStrMap[key] + this.formatTargetValue(key);
    }, "") + beforeStrMap.rest;
  }
  findNearSelectionTarget(selectionStart) {
    const indexMap = this.getIndexMap();
    let prevDiff = Infinity;
    let target = "";
    forEach(indexMap, ([std, end], key) => {
      const diff = Math.min(
        Math.abs(selectionStart - std),
        Math.abs(end - selectionStart)
      );
      if (target && diff > prevDiff) return;
      prevDiff = diff;
      target = key;
    });
    return { target, indexMap };
  }
  /** Select first editable text field */
  handleFocus(event) {
    event.preventDefault();
    if (this.options.readOnly) return;
    this.$inputBox.classList.add("--active");
    if (get(event.currentTarget, "isPointerDown")) {
      return assignIn(event.currentTarget, { isPointerDown: false });
    }
    const { target, indexMap } = this.findNearSelectionTarget(0);
    this.editState = { target, indexMap, count: 0 };
    event.currentTarget.setSelectionRange(...indexMap[target]);
  }
  /** Select near eidtable text field */
  handlePointerUp(event) {
    var _a;
    event.preventDefault();
    if (this.options.readOnly) return;
    assignIn(event.currentTarget, { isPointerDown: false });
    const selectionStart = (_a = event.currentTarget.selectionStart) != null ? _a : 0;
    const { target, indexMap } = this.findNearSelectionTarget(selectionStart);
    this.editState = { target, indexMap, count: 0 };
    event.currentTarget.setSelectionRange(...indexMap[target]);
  }
  /** Make input[type="text"] like input[type="date"] */
  handleKeydownInput(event) {
    if (this.options.readOnly || event.isComposing) return;
    if (!event.ctrlKey || event.key !== "v") event.preventDefault();
    const num = Number(event.key);
    if (!/\d/g.test(event.key)) {
      event.preventDefault();
      event.stopPropagation();
      event.stopImmediatePropagation();
      return;
    }
    const { value } = this.$input;
    const { formatStrMap, valueMap, keyOrders } = this.inputState;
    const { indexMap, target } = this.editState;
    const indexes = indexMap[target];
    const maxCount = target !== "year" ? 2 : formatStrMap[target].length;
    this.editState.count++;
    if (target === "year") {
      valueMap[target] = this.editState.count === 1 ? num : Number(`${valueMap[target]}${num}`);
    } else if (target === "month") {
      if (this.editState.count === 1) {
        valueMap[target] = num;
        if (num > 1) this.editState.count++;
      } else {
        valueMap[target] = num < 3 ? Number(valueMap[target] + `${num}`) : num;
      }
    } else {
      if (this.editState.count === 1) {
        valueMap[target] = num;
        const max = valueMap.month && valueMap.month !== 2 ? 3 : 2;
        if (num > max) this.editState.count++;
      } else {
        const max = new Date(valueMap.year, valueMap.month, 0, 12).getDate();
        const value2 = Number(valueMap[target] + `${num}`);
        valueMap[target] = value2 > max ? num : value2;
      }
    }
    const selectionText = value.slice(...indexes);
    const changeText = this.formatTargetValue(target);
    const textDiff = selectionText.length - changeText.length;
    if (textDiff !== 0) {
      this.editState.indexMap = this.getIndexMap();
    }
    this.$input.value = value.slice(0, indexes[0]) + changeText + value.slice(indexes[1]);
    if (this.editState.count >= maxCount) {
      const nextIdx = keyOrders.findIndex((item) => item === target) + 1;
      const keyOrderSize = keyOrders.length - 1;
      const nextTarget = keyOrders[nextIdx > keyOrderSize ? keyOrderSize : nextIdx];
      this.editState.target = nextTarget;
      this.editState.count = 0;
    }
    this.$input.setSelectionRange(
      ...this.editState.indexMap[this.editState.target]
    );
  }
  /** Change input value, If paste text is match with format */
  handlePaste(event) {
    var _a, _b, _c;
    if (this.options.readOnly) return;
    if (document.activeElement !== event.currentTarget) return;
    event.preventDefault();
    let paste = ((_b = (_a = (event == null ? void 0 : event.clipboardData) || window.clipboardData) == null ? void 0 : _a.getData(
      "text"
    )) != null ? _b : "").trim();
    const { formatStrMap, beforeStrMap, keyOrders } = this.inputState;
    const formatMonthSize = formatStrMap.month.length;
    if (formatMonthSize > 2) {
      const monthStrs = this.instance.converter.locale[formatMonthSize === 3 ? "monthsShort" : "months"];
      monthStrs.forEach(
        (item, idx) => paste = paste.replace(item, `${idx + 1}`)
      );
    }
    let items = [];
    let prevIdx = 0;
    paste.replace(IGNORE_REGEXP, (match, idx) => {
      const length = match.length;
      if (length === 0) return "";
      items.push(paste.slice(prevIdx, idx));
      items.push(match);
      prevIdx = idx + length;
      return "";
    });
    items.push(paste.slice(prevIdx));
    items = items.filter((item) => item.length);
    const data = { year: "", month: "", day: "" };
    let itemPos = 0;
    for (const key of keyOrders) {
      const beforeStr = beforeStrMap[key];
      if (beforeStr.length) {
        if (beforeStr !== items[itemPos]) return;
        itemPos++;
      }
      const item = items[itemPos];
      itemPos++;
      if (item === formatStrMap[key] || isNaN(Number(item))) return;
      data[key] = item;
    }
    if (((_c = items[itemPos]) != null ? _c : "") !== beforeStrMap.rest || items.at(itemPos + 1))
      return;
    if (!isDateLike(`${data.year}-${data.month}-${data.day}`)) return;
    forEach(
      data,
      (value, key) => this.inputState.valueMap[key] = value
    );
    event.currentTarget.value = this.convertInputStateValue();
  }
  /** Reset Input value to formatted datepicker selected date */
  resetDate() {
    const { converter } = this.instance;
    const selectedDate = this.dp.selectedDate;
    this.$input.value = converter.format(selectedDate, this.displayFormat);
  }
  /** Set datepicker's selected date to input value date */
  setToInputDate() {
    const date = this.inputDate;
    if (this.options.readOnly) return;
    if (!isDateLike(date)) return this.resetDate();
    this.dp.setSelectedDate(date);
    this.dp.setViewDate(date);
  }
  /** Set date & reset input state */
  handleBlur(event) {
    this.$inputBox.classList.remove("--active");
    if (!get(event.currentTarget, "dpDisableInputBlur")) this.setToInputDate();
    assignIn(event.currentTarget, { dpDisableInputBlur: false });
  }
  /** Escape input or submit input value */
  handleKeydown(event) {
    if (this.options.readOnly) return;
    const isEnter = event.key === "Enter";
    const isEscape = event.key === "Escape";
    if (isEnter || isEscape) {
      event.preventDefault();
      assignIn(this.$input, { dpDisableInputBlur: true });
      this.$input.blur();
      isEnter ? this.setToInputDate() : this.resetDate();
      return false;
    }
    if (event.key === "Tab") {
      event.preventDefault();
      const { keyOrders } = this.inputState;
      const { target, indexMap } = this.editState;
      const adder = event.shiftKey ? -1 : 1;
      const keySize = keyOrders.length;
      let nextIdx = keyOrders.findIndex((item) => item === target) + adder;
      if (nextIdx < 0) nextIdx = keySize + nextIdx;
      else if (nextIdx >= keySize) nextIdx = nextIdx % keySize;
      const nextTarget = keyOrders[nextIdx];
      this.editState.count = 0;
      this.editState.target = nextTarget;
      this.$input.setSelectionRange(...indexMap[nextTarget]);
      return false;
    }
  }
}

class PickerEvent {
  constructor({ event, ...props } = {}) {
    if (event) this.assignData({ nativeEvent: event });
    this.assignData(props);
  }
  assignData(data) {
    assign(this, data);
  }
}

function createEventManager(datepicker, options) {
  const listenerMap = {};
  const manager = {
    on: (eventName, listener) => {
      const listeners = listenerMap[eventName];
      isArray(listeners) ? listeners.push(listener) : listenerMap[eventName] = [listener];
    },
    off: (eventName, listener) => {
      const listeners = listenerMap[eventName];
      if (!isArray(listener)) return;
      const index = listeners.findIndex((item) => item === listener);
      if (index > -1) {
        listenerMap[eventName] = listeners.slice(0, index).concat(listener.slice(index + 1));
      }
    },
    trigger: (eventName, eventProps) => {
      const listeners = listenerMap[eventName];
      if (!isArray(listeners)) return;
      const pickerEvent = new PickerEvent({ ...eventProps, datepicker });
      return listeners.map((listener) => {
        return listener(pickerEvent);
      });
    }
  };
  OPTION_EVENT_KEYS.forEach((key) => {
    var _a;
    const item = get(options, key);
    const eventName = ((_a = key.at(2)) == null ? void 0 : _a.toLowerCase()) + key.slice(3);
    if (isFunction(item)) manager.on(eventName, item);
  });
  return manager;
}

function isView(view) {
  return isString(view) && VIEW_LIST.includes(view);
}
function checkView(view, minView) {
  return VIEW_ORDER[view] >= VIEW_ORDER[minView];
}
function inValid(key, message) {
  error(`Invalid options`, `options.${key} should be ${message}`);
}
const VALIDATION_MAP = assign(
  {},
  reduce(
    DEFUALT_OPTIONS,
    (obj, value, key) => {
      const type = typeof value;
      obj[key] = (val) => {
        if (typeof val === type) return;
        inValid(key, type);
      };
      return obj;
    },
    {}
  ),
  ["minDate", "maxDate", "selectedDate"].reduce((obj, item) => {
    obj[item] = (val) => {
      if (isUndefined(val) || isDate(val)) return;
      inValid(item, "Date");
    };
    return obj;
  }, {}),
  ["className", "placeHolder"].reduce((obj, item) => {
    obj[item] = (val) => {
      if (isUndefined(val) || isString(val)) return;
      inValid(item, "string");
    };
    return obj;
  }, {}),
  ["view", "minView"].reduce((obj, item) => {
    obj[item] = (val) => {
      if (isUndefined(val) || VIEW_LIST.includes(val)) return;
      inValid(item, VIEW_LIST.join(" | "));
    };
    return obj;
  }, {}),
  OPTION_EVENT_KEYS.reduce((obj, item) => {
    obj[item] = (val) => {
      if (isUndefined(val) || isFunction(val)) return;
      inValid(item, "function");
    };
    return obj;
  }, {}),
  {
    position: (val) => {
      if (isUndefined(val)) return;
      if (isString(val)) {
        const [pos, edge, ...ele] = val.split("-");
        if (!ele.length && ["top", "bottom", "left", "right"].includes(pos) && (isUndefined(edge) || ["start", "end"].includes(edge)))
          return;
      }
      inValid("position", "Position");
    },
    theme: (val) => {
      if (isString(val) && (val === "light" || val === "dark")) return;
      inValid("theme", "light | dark");
    },
    size: (val) => {
      if (isString(val) && (val === "small" || val === "normal")) return;
      inValid("theme", "small | normal");
    },
    titleFormat: (val) => {
      if (isUndefined(val)) return;
      if (!isObject(val))
        return inValid(
          "titleFormat",
          "{ days?: string; months?: string; years?: string }"
        );
      forEach(DEFAULT_TITLE_FORMAT, (_, key) => {
        const matcher = ["string", "undefined"];
        if (matcher.includes(typeof val[key])) return;
        inValid(`titleFormat.${key}`, matcher.join(" | "));
      });
    },
    buttons: (val) => {
      if (isUndefined(val)) return;
      const buttons = isArray(val) ? val : [val];
      const isError = !buttons.every((button) => {
        if (isString(button) && BUTTON_PRESETS.includes(button)) return true;
        if (isPlainObject(button)) {
          const { className, id, dataset, attrs, innerHTML } = button;
          const isValidString = [className, id, innerHTML].every(
            (item) => isUndefined(item) || isString(item)
          );
          const isValidObject = [dataset, attrs].every(
            (obj) => isUndefined(obj) || values(obj).every((item) => isString(item))
          );
          return isValidString && isValidObject;
        }
        return false;
      });
      if (isError)
        error(
          "buttons",
          `value must be in ${BUTTON_PRESETS.map((item) => "`" + item + "`").join(" | ")} or button-options object`
        );
    }
  }
);
function checkSchema(options) {
  const opt = assign({}, DEFUALT_OPTIONS, options);
  if (opt.titleFormat === void 0) opt.titleFormat = {};
  if (isPlainObject(opt.titleFormat)) {
    forEach(DEFAULT_TITLE_FORMAT, (value, key) => {
      const titleFormat = opt.titleFormat;
      if (isNullish(titleFormat[key])) titleFormat[key] = value;
    });
  }
  forEach(opt, (value, key) => {
    const validator = VALIDATION_MAP[key];
    if (!validator) return;
    validator(value);
  });
  if (!checkView(opt.view, opt.minView)) opt.view = opt.minView;
  const { minDate, maxDate } = opt;
  const min = !minDate ? new Date(MIN_DATE) : new Date(Math.max(Number(MIN_DATE), Number(new Date(minDate))));
  const max = !maxDate ? new Date(MAX_DATE) : new Date(Math.min(Number(new Date(maxDate)), Number(maxDate)));
  min.setHours(0), min.setMinutes(0), min.setSeconds(1);
  max.setHours(23), max.setMinutes(59), max.setSeconds(59);
  opt.minDate = min;
  opt.maxDate = max;
  return opt;
}

const ADDER_MAP = {
  days: {
    ArrowLeft: -1,
    ArrowRight: 1,
    ArrowDown: 7,
    ArrowUp: -7
  },
  months: {
    ArrowLeft: -1,
    ArrowRight: 1,
    ArrowDown: 3,
    ArrowUp: -3
  },
  years: {
    ArrowLeft: -1,
    ArrowRight: 1,
    ArrowDown: 4,
    ArrowUp: -4
  }
};
const ADD_IDX_MAP = {
  days: 2,
  months: 1,
  years: 0
};
const AVAILABLE_KEYS = ["ArrowLeft", "ArrowRight", "ArrowUp", "ArrowDown"];
const isArrowKey = (key) => AVAILABLE_KEYS.includes(key);
function createShortcutsHandler(dp) {
  const handleShrotchust = (event) => {
    const { ctrlKey, shiftKey, altKey, key } = event;
    if (key === "Escape") {
      event.preventDefault();
      return dp.hide();
    }
    if (event.key === "PageUp" || event.key === "PageDown") {
      event.preventDefault();
      event.stopPropagation();
      const adder = event.key === "PageUp" ? 1 : -1;
      const nextIdx = VIEW_LIST.findIndex((item) => item === dp.currentView) + adder;
      if (between(nextIdx, 0, 2)) {
        dp.setCurrentView(VIEW_LIST[nextIdx]);
      }
      dp.$datepicker.focus();
      return;
    }
    if (!ctrlKey && !shiftKey && !altKey && event.key === "Enter") {
      event.preventDefault();
      const { focusDate: focusDate2, currentView: currentView2 } = dp;
      if (!focusDate2) return;
      const [year2, monthindex2, day2] = parseDate(focusDate2);
      let sufix = `[data-year="${year2}"]`;
      if (currentView2 !== "years") {
        sufix += `[data-monthindex="${monthindex2}"]`;
        if (currentView2 !== "months") sufix += `[data-day="${day2}"]`;
      }
      const $cell = find$(`.${cn("cell")}${sufix}`, dp.$datepicker);
      if ($cell) $cell.click();
      return;
    }
    if (!isArrowKey(key)) return;
    event.preventDefault();
    const { focusDate, viewDate, selectedDate, currentView } = dp;
    let date = focusDate;
    if (!date) {
      date = viewDate;
      if (selectedDate && selectedDate.getFullYear() === date.getFullYear() && selectedDate.getMonth() === date.getMonth()) {
        date = selectedDate;
      } else {
        if (key === "ArrowLeft" || key === "ArrowUp")
          date = new Date(date.getFullYear(), date.getMonth() + 1, 0);
        else date = new Date(date.getFullYear(), date.getMonth(), 1);
      }
    }
    const parsed = parseDate(date);
    const [year, monthindex, day] = parsed;
    const [startYear, endYear] = decade(date);
    const isUp = key === "ArrowRight" || key === "ArrowDown";
    if (!ctrlKey && !shiftKey && !altKey) {
      const params = [...parsed];
      params[ADD_IDX_MAP[currentView]] += ADDER_MAP[currentView][key];
      if (currentView === "years") {
        if (params[0] < startYear - 1) params[0] += 2;
        if (params[0] > endYear + 1) params[0] -= 2;
      }
      const nextDate = new Date(...params);
      if (currentView === "days" && nextDate.getMonth() !== monthindex || currentView === "months" && nextDate.getMonth() !== year || currentView === "years" && between(year, startYear, endYear))
        dp.setViewDate(nextDate);
      dp.setFocusDate(nextDate);
      return;
    } else if (ctrlKey && !shiftKey && !altKey) {
      const adder = isUp ? 1 : -1;
      const nextDate = new Date(year, monthindex + adder, day);
      dp.setFocusDate(nextDate);
      dp.setViewDate(nextDate);
    } else if (!ctrlKey && shiftKey && !altKey) {
      const adder = isUp ? 1 : -1;
      const nextDate = new Date(year + adder, monthindex, day);
      dp.setFocusDate(nextDate);
      dp.setViewDate(nextDate);
    } else if (!ctrlKey && !shiftKey && altKey) {
      let nextYear = year + (isUp ? 10 : -10);
      if (nextYear < 100) nextYear = 1e4 - 100 + nextYear;
      if (nextYear > 9999) nextYear = 100 + nextYear - 1e4;
      const nextDate = new Date(nextYear, monthindex, day);
      dp.setFocusDate(nextDate);
      dp.setViewDate(nextDate);
    }
    dp.$datepicker.focus();
  };
  return handleShrotchust;
}

const locale = {
  name: "en",
  weekdays: [
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday"
  ],
  months: [
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December"
  ],
  weekStart: 1,
  weekdaysShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
  monthsShort: [
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec"
  ],
  weekdaysMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"],
  formats: { time: "HH:mm", date: "DD/MM/YYYY" },
  placeholder: "Choose date"
};

const getZone = (date) => {
  const zoneOffset = -(-Math.round(date.getTimezoneOffset() / 15) * 15);
  const offset = Math.abs(zoneOffset);
  const operator = zoneOffset <= 0 ? "+" : "-";
  const offsetHours = Math.floor(offset / 60).toString().padStart(2, "0");
  const offsetMinutes = (offset % 60).toString().padStart(2, "0");
  return `${operator}${offsetHours}:${offsetMinutes}`;
};
function craeteConverter({
  locale: locale$1 = locale,
  dateFormat
} = {}) {
  const hour = (hours) => hours % 12 || 12;
  const meridiem = (hours) => hours < 12 ? "AM" : "PM";
  const converter = {
    get locale() {
      return locale$1;
    },
    get weekIndexes() {
      const { weekStart = 0 } = locale$1;
      return range(weekStart, 7).map((item) => item % 7);
    },
    /**
     * Format date
     * @param {any} value The value to format.
     * @param {string} formatStr The format string for formating value.
     * @returns {string} Returns formatted value.
     */
    format(value, formatStr = DEFAULT_FORMAT) {
      if (!isDateLike(value)) {
        return formatStr.replace(FORMAT_REGEXP, (match, escapeStr) => {
          if (escapeStr) return escapeStr;
          return match;
        });
      }
      const date = new Date(value);
      const Y = date.getFullYear();
      const M = date.getMonth();
      const D = date.getDate();
      const W = date.getDay();
      const H = date.getHours();
      const m = date.getMinutes();
      const s = date.getSeconds();
      const ms = date.getMilliseconds();
      const [d1, d2] = decade(date);
      return formatStr.replace(FORMAT_REGEXP, (match, escapeStr) => {
        if (escapeStr) return escapeStr;
        switch (match) {
          case "YY1":
            return d1.toString().slice(-2);
          case "YY2":
            return d2.toString().slice(-2);
          case "YY":
            return Y.toString().slice(-2);
          case "YYYY1":
            return d1;
          case "YYYY2":
            return d2;
          case "YYYY":
            return Y.toString().padStart(4, "0");
          case "M":
            return M + 1;
          case "MM":
            return (M + 1).toString().padStart(2, "0");
          case "MMM":
            return this.localeMonth(M, true);
          case "MMMM":
            return this.localeMonth(M);
          case "D":
            return D;
          case "DD":
            return D.toString().padStart(2, "0");
          case "d":
            return W;
          case "dd":
            return this.localeWeekday(W, "min");
          case "ddd":
            return this.localeWeekday(W, "short");
          case "dddd":
            return this.localeWeekday(W);
          case "H":
            return H;
          case "HH":
            return H.toString().padStart(2, "0");
          case "h":
            return hour(H);
          case "hh":
            return hour(H).toString().padStart(2, "0");
          case "a":
            return meridiem(H).toLowerCase();
          case "A":
            return meridiem(H);
          case "m":
            return m;
          case "mm":
            return m.toString().padStart(2, "0");
          case "s":
            return s;
          case "ss":
            return s.toString().padStart(2, "0");
          case "sss":
            return ms.toString().padStart(3, "0");
          case "Z":
            return getZone(date);
          case "ZZ":
            return getZone(date).replace(":", "");
          default:
            return "";
        }
      });
    },
    /**
     * Convert date to locale time format
     * @param {any} value The value to convert
     * @returns {string} Returns formatted time
     */
    time(value) {
      return this.format(value, locale$1.formats.time);
    },
    /**
     * Convert date to options date format (defaults locale date format)
     * @param {any} value The value to convert.
     * @returns {string} Returns formatted date.
     */
    date(value) {
      return this.format(value, dateFormat != null ? dateFormat : locale$1.formats.date);
    },
    /**
     * Convert monthindex to locale format month string
     * @param {number} monthIndex The value to convert
     * @param {boolean} [short] If `true`, it return short string of month
     * @returns {string} Returns month string
     */
    localeMonth(monthIndex, short) {
      const monthStr = locale$1.months[monthIndex];
      if (!short) return monthStr;
      return locale$1.monthsShort[monthIndex] || monthStr.slice(0, 3);
    },
    /**
     * Convert weekdayindex to locale format weekday string
     * @param {number} weekdayIndex The value to convert
     * @param {'short' | 'min'} [options] The options to convert value
     * @returns {string} Returns weekday string
     */
    localeWeekday(weekdayIndex, options) {
      if (options === "short") return locale$1.weekdaysShort[weekdayIndex];
      if (options === "min") return locale$1.weekdaysMin[weekdayIndex];
      return locale$1.weekdays[weekdayIndex];
    }
  };
  return converter;
}

function createStore({ minView }) {
  const date = observable(null);
  const currentView = observable(minView || "days");
  const viewDate = observable(/* @__PURE__ */ new Date());
  const focusDate = observable(null);
  const viewState = observable(() => {
    const ud = viewDate();
    const cu = currentView();
    let args = parseDate(ud).slice(0, 3 - VIEW_ORDER[cu]);
    args = args.length ? args : decade(ud);
    return `${cu}__${args.join("__")}`;
  });
  const store = {
    state: {
      date,
      currentView,
      viewDate,
      focusDate,
      viewState
    },
    get currentView() {
      return currentView();
    },
    get viewDate() {
      return viewDate();
    },
    get selectedDate() {
      const selectedDate = date();
      return isNullish(selectedDate) ? null : new Date(selectedDate);
    },
    setSelectedDate: (value) => {
      date(!isDateLike(value) ? null : new Date(value));
    },
    setCurrentView: (value) => {
      let nextView = isView(value) ? value : minView;
      if (!checkView(value, minView)) nextView = minView;
      currentView(nextView);
    },
    setViewDate: (value) => {
      if (!isDateLike(value))
        return console.log(`This viewDate is invalid date - ${value}`);
      viewDate(new Date(value));
    }
  };
  return store;
}

function createInstance(options) {
  const { minView, locale, dateFormat } = options;
  const converter = craeteConverter({ locale, dateFormat });
  const store = createStore({ minView });
  const instance = { converter, store };
  return instance;
}

/**
 * Custom positioning reference element.
 * @see https://floating-ui.com/docs/virtual-elements
 */

const min = Math.min;
const max = Math.max;
const round = Math.round;
const floor = Math.floor;
const createCoords = v => ({
  x: v,
  y: v
});
const oppositeSideMap = {
  left: 'right',
  right: 'left',
  bottom: 'top',
  top: 'bottom'
};
const oppositeAlignmentMap = {
  start: 'end',
  end: 'start'
};
function evaluate(value, param) {
  return typeof value === 'function' ? value(param) : value;
}
function getSide(placement) {
  return placement.split('-')[0];
}
function getAlignment(placement) {
  return placement.split('-')[1];
}
function getOppositeAxis(axis) {
  return axis === 'x' ? 'y' : 'x';
}
function getAxisLength(axis) {
  return axis === 'y' ? 'height' : 'width';
}
function getSideAxis(placement) {
  return ['top', 'bottom'].includes(getSide(placement)) ? 'y' : 'x';
}
function getAlignmentAxis(placement) {
  return getOppositeAxis(getSideAxis(placement));
}
function getAlignmentSides(placement, rects, rtl) {
  if (rtl === void 0) {
    rtl = false;
  }
  const alignment = getAlignment(placement);
  const alignmentAxis = getAlignmentAxis(placement);
  const length = getAxisLength(alignmentAxis);
  let mainAlignmentSide = alignmentAxis === 'x' ? alignment === (rtl ? 'end' : 'start') ? 'right' : 'left' : alignment === 'start' ? 'bottom' : 'top';
  if (rects.reference[length] > rects.floating[length]) {
    mainAlignmentSide = getOppositePlacement(mainAlignmentSide);
  }
  return [mainAlignmentSide, getOppositePlacement(mainAlignmentSide)];
}
function getExpandedPlacements(placement) {
  const oppositePlacement = getOppositePlacement(placement);
  return [getOppositeAlignmentPlacement(placement), oppositePlacement, getOppositeAlignmentPlacement(oppositePlacement)];
}
function getOppositeAlignmentPlacement(placement) {
  return placement.replace(/start|end/g, alignment => oppositeAlignmentMap[alignment]);
}
function getSideList(side, isStart, rtl) {
  const lr = ['left', 'right'];
  const rl = ['right', 'left'];
  const tb = ['top', 'bottom'];
  const bt = ['bottom', 'top'];
  switch (side) {
    case 'top':
    case 'bottom':
      if (rtl) return isStart ? rl : lr;
      return isStart ? lr : rl;
    case 'left':
    case 'right':
      return isStart ? tb : bt;
    default:
      return [];
  }
}
function getOppositeAxisPlacements(placement, flipAlignment, direction, rtl) {
  const alignment = getAlignment(placement);
  let list = getSideList(getSide(placement), direction === 'start', rtl);
  if (alignment) {
    list = list.map(side => side + "-" + alignment);
    if (flipAlignment) {
      list = list.concat(list.map(getOppositeAlignmentPlacement));
    }
  }
  return list;
}
function getOppositePlacement(placement) {
  return placement.replace(/left|right|bottom|top/g, side => oppositeSideMap[side]);
}
function expandPaddingObject(padding) {
  return {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
    ...padding
  };
}
function getPaddingObject(padding) {
  return typeof padding !== 'number' ? expandPaddingObject(padding) : {
    top: padding,
    right: padding,
    bottom: padding,
    left: padding
  };
}
function rectToClientRect(rect) {
  return {
    ...rect,
    top: rect.y,
    left: rect.x,
    right: rect.x + rect.width,
    bottom: rect.y + rect.height
  };
}

function computeCoordsFromPlacement(_ref, placement, rtl) {
  let {
    reference,
    floating
  } = _ref;
  const sideAxis = getSideAxis(placement);
  const alignmentAxis = getAlignmentAxis(placement);
  const alignLength = getAxisLength(alignmentAxis);
  const side = getSide(placement);
  const isVertical = sideAxis === 'y';
  const commonX = reference.x + reference.width / 2 - floating.width / 2;
  const commonY = reference.y + reference.height / 2 - floating.height / 2;
  const commonAlign = reference[alignLength] / 2 - floating[alignLength] / 2;
  let coords;
  switch (side) {
    case 'top':
      coords = {
        x: commonX,
        y: reference.y - floating.height
      };
      break;
    case 'bottom':
      coords = {
        x: commonX,
        y: reference.y + reference.height
      };
      break;
    case 'right':
      coords = {
        x: reference.x + reference.width,
        y: commonY
      };
      break;
    case 'left':
      coords = {
        x: reference.x - floating.width,
        y: commonY
      };
      break;
    default:
      coords = {
        x: reference.x,
        y: reference.y
      };
  }
  switch (getAlignment(placement)) {
    case 'start':
      coords[alignmentAxis] -= commonAlign * (rtl && isVertical ? -1 : 1);
      break;
    case 'end':
      coords[alignmentAxis] += commonAlign * (rtl && isVertical ? -1 : 1);
      break;
  }
  return coords;
}

/**
 * Computes the `x` and `y` coordinates that will place the floating element
 * next to a given reference element.
 *
 * This export does not have any `platform` interface logic. You will need to
 * write one for the platform you are using Floating UI with.
 */
const computePosition$1 = async (reference, floating, config) => {
  const {
    placement = 'bottom',
    strategy = 'absolute',
    middleware = [],
    platform
  } = config;
  const validMiddleware = middleware.filter(Boolean);
  const rtl = await (platform.isRTL == null ? void 0 : platform.isRTL(floating));
  let rects = await platform.getElementRects({
    reference,
    floating,
    strategy
  });
  let {
    x,
    y
  } = computeCoordsFromPlacement(rects, placement, rtl);
  let statefulPlacement = placement;
  let middlewareData = {};
  let resetCount = 0;
  for (let i = 0; i < validMiddleware.length; i++) {
    const {
      name,
      fn
    } = validMiddleware[i];
    const {
      x: nextX,
      y: nextY,
      data,
      reset
    } = await fn({
      x,
      y,
      initialPlacement: placement,
      placement: statefulPlacement,
      strategy,
      middlewareData,
      rects,
      platform,
      elements: {
        reference,
        floating
      }
    });
    x = nextX != null ? nextX : x;
    y = nextY != null ? nextY : y;
    middlewareData = {
      ...middlewareData,
      [name]: {
        ...middlewareData[name],
        ...data
      }
    };
    if (reset && resetCount <= 50) {
      resetCount++;
      if (typeof reset === 'object') {
        if (reset.placement) {
          statefulPlacement = reset.placement;
        }
        if (reset.rects) {
          rects = reset.rects === true ? await platform.getElementRects({
            reference,
            floating,
            strategy
          }) : reset.rects;
        }
        ({
          x,
          y
        } = computeCoordsFromPlacement(rects, statefulPlacement, rtl));
      }
      i = -1;
    }
  }
  return {
    x,
    y,
    placement: statefulPlacement,
    strategy,
    middlewareData
  };
};

/**
 * Resolves with an object of overflow side offsets that determine how much the
 * element is overflowing a given clipping boundary on each side.
 * - positive = overflowing the boundary by that number of pixels
 * - negative = how many pixels left before it will overflow
 * - 0 = lies flush with the boundary
 * @see https://floating-ui.com/docs/detectOverflow
 */
async function detectOverflow(state, options) {
  var _await$platform$isEle;
  if (options === void 0) {
    options = {};
  }
  const {
    x,
    y,
    platform,
    rects,
    elements,
    strategy
  } = state;
  const {
    boundary = 'clippingAncestors',
    rootBoundary = 'viewport',
    elementContext = 'floating',
    altBoundary = false,
    padding = 0
  } = evaluate(options, state);
  const paddingObject = getPaddingObject(padding);
  const altContext = elementContext === 'floating' ? 'reference' : 'floating';
  const element = elements[altBoundary ? altContext : elementContext];
  const clippingClientRect = rectToClientRect(await platform.getClippingRect({
    element: ((_await$platform$isEle = await (platform.isElement == null ? void 0 : platform.isElement(element))) != null ? _await$platform$isEle : true) ? element : element.contextElement || (await (platform.getDocumentElement == null ? void 0 : platform.getDocumentElement(elements.floating))),
    boundary,
    rootBoundary,
    strategy
  }));
  const rect = elementContext === 'floating' ? {
    ...rects.floating,
    x,
    y
  } : rects.reference;
  const offsetParent = await (platform.getOffsetParent == null ? void 0 : platform.getOffsetParent(elements.floating));
  const offsetScale = (await (platform.isElement == null ? void 0 : platform.isElement(offsetParent))) ? (await (platform.getScale == null ? void 0 : platform.getScale(offsetParent))) || {
    x: 1,
    y: 1
  } : {
    x: 1,
    y: 1
  };
  const elementClientRect = rectToClientRect(platform.convertOffsetParentRelativeRectToViewportRelativeRect ? await platform.convertOffsetParentRelativeRectToViewportRelativeRect({
    elements,
    rect,
    offsetParent,
    strategy
  }) : rect);
  return {
    top: (clippingClientRect.top - elementClientRect.top + paddingObject.top) / offsetScale.y,
    bottom: (elementClientRect.bottom - clippingClientRect.bottom + paddingObject.bottom) / offsetScale.y,
    left: (clippingClientRect.left - elementClientRect.left + paddingObject.left) / offsetScale.x,
    right: (elementClientRect.right - clippingClientRect.right + paddingObject.right) / offsetScale.x
  };
}

/**
 * Optimizes the visibility of the floating element by flipping the `placement`
 * in order to keep it in view when the preferred placement(s) will overflow the
 * clipping boundary. Alternative to `autoPlacement`.
 * @see https://floating-ui.com/docs/flip
 */
const flip$1 = function (options) {
  if (options === void 0) {
    options = {};
  }
  return {
    name: 'flip',
    options,
    async fn(state) {
      var _middlewareData$arrow, _middlewareData$flip;
      const {
        placement,
        middlewareData,
        rects,
        initialPlacement,
        platform,
        elements
      } = state;
      const {
        mainAxis: checkMainAxis = true,
        crossAxis: checkCrossAxis = true,
        fallbackPlacements: specifiedFallbackPlacements,
        fallbackStrategy = 'bestFit',
        fallbackAxisSideDirection = 'none',
        flipAlignment = true,
        ...detectOverflowOptions
      } = evaluate(options, state);

      // If a reset by the arrow was caused due to an alignment offset being
      // added, we should skip any logic now since `flip()` has already done its
      // work.
      // https://github.com/floating-ui/floating-ui/issues/2549#issuecomment-1719601643
      if ((_middlewareData$arrow = middlewareData.arrow) != null && _middlewareData$arrow.alignmentOffset) {
        return {};
      }
      const side = getSide(placement);
      const isBasePlacement = getSide(initialPlacement) === initialPlacement;
      const rtl = await (platform.isRTL == null ? void 0 : platform.isRTL(elements.floating));
      const fallbackPlacements = specifiedFallbackPlacements || (isBasePlacement || !flipAlignment ? [getOppositePlacement(initialPlacement)] : getExpandedPlacements(initialPlacement));
      if (!specifiedFallbackPlacements && fallbackAxisSideDirection !== 'none') {
        fallbackPlacements.push(...getOppositeAxisPlacements(initialPlacement, flipAlignment, fallbackAxisSideDirection, rtl));
      }
      const placements = [initialPlacement, ...fallbackPlacements];
      const overflow = await detectOverflow(state, detectOverflowOptions);
      const overflows = [];
      let overflowsData = ((_middlewareData$flip = middlewareData.flip) == null ? void 0 : _middlewareData$flip.overflows) || [];
      if (checkMainAxis) {
        overflows.push(overflow[side]);
      }
      if (checkCrossAxis) {
        const sides = getAlignmentSides(placement, rects, rtl);
        overflows.push(overflow[sides[0]], overflow[sides[1]]);
      }
      overflowsData = [...overflowsData, {
        placement,
        overflows
      }];

      // One or more sides is overflowing.
      if (!overflows.every(side => side <= 0)) {
        var _middlewareData$flip2, _overflowsData$filter;
        const nextIndex = (((_middlewareData$flip2 = middlewareData.flip) == null ? void 0 : _middlewareData$flip2.index) || 0) + 1;
        const nextPlacement = placements[nextIndex];
        if (nextPlacement) {
          // Try next placement and re-run the lifecycle.
          return {
            data: {
              index: nextIndex,
              overflows: overflowsData
            },
            reset: {
              placement: nextPlacement
            }
          };
        }

        // First, find the candidates that fit on the mainAxis side of overflow,
        // then find the placement that fits the best on the main crossAxis side.
        let resetPlacement = (_overflowsData$filter = overflowsData.filter(d => d.overflows[0] <= 0).sort((a, b) => a.overflows[1] - b.overflows[1])[0]) == null ? void 0 : _overflowsData$filter.placement;

        // Otherwise fallback.
        if (!resetPlacement) {
          switch (fallbackStrategy) {
            case 'bestFit':
              {
                var _overflowsData$map$so;
                const placement = (_overflowsData$map$so = overflowsData.map(d => [d.placement, d.overflows.filter(overflow => overflow > 0).reduce((acc, overflow) => acc + overflow, 0)]).sort((a, b) => a[1] - b[1])[0]) == null ? void 0 : _overflowsData$map$so[0];
                if (placement) {
                  resetPlacement = placement;
                }
                break;
              }
            case 'initialPlacement':
              resetPlacement = initialPlacement;
              break;
          }
        }
        if (placement !== resetPlacement) {
          return {
            reset: {
              placement: resetPlacement
            }
          };
        }
      }
      return {};
    }
  };
};

function getNodeName(node) {
  if (isNode(node)) {
    return (node.nodeName || '').toLowerCase();
  }
  // Mocked nodes in testing environments may not be instances of Node. By
  // returning `#document` an infinite loop won't occur.
  // https://github.com/floating-ui/floating-ui/issues/2317
  return '#document';
}
function getWindow(node) {
  var _node$ownerDocument;
  return (node == null || (_node$ownerDocument = node.ownerDocument) == null ? void 0 : _node$ownerDocument.defaultView) || window;
}
function getDocumentElement(node) {
  var _ref;
  return (_ref = (isNode(node) ? node.ownerDocument : node.document) || window.document) == null ? void 0 : _ref.documentElement;
}
function isNode(value) {
  return value instanceof Node || value instanceof getWindow(value).Node;
}
function isElement(value) {
  return value instanceof Element || value instanceof getWindow(value).Element;
}
function isHTMLElement(value) {
  return value instanceof HTMLElement || value instanceof getWindow(value).HTMLElement;
}
function isShadowRoot(value) {
  // Browsers without `ShadowRoot` support.
  if (typeof ShadowRoot === 'undefined') {
    return false;
  }
  return value instanceof ShadowRoot || value instanceof getWindow(value).ShadowRoot;
}
function isOverflowElement(element) {
  const {
    overflow,
    overflowX,
    overflowY,
    display
  } = getComputedStyle(element);
  return /auto|scroll|overlay|hidden|clip/.test(overflow + overflowY + overflowX) && !['inline', 'contents'].includes(display);
}
function isTableElement(element) {
  return ['table', 'td', 'th'].includes(getNodeName(element));
}
function isContainingBlock(element) {
  const webkit = isWebKit();
  const css = getComputedStyle(element);

  // https://developer.mozilla.org/en-US/docs/Web/CSS/Containing_block#identifying_the_containing_block
  return css.transform !== 'none' || css.perspective !== 'none' || (css.containerType ? css.containerType !== 'normal' : false) || !webkit && (css.backdropFilter ? css.backdropFilter !== 'none' : false) || !webkit && (css.filter ? css.filter !== 'none' : false) || ['transform', 'perspective', 'filter'].some(value => (css.willChange || '').includes(value)) || ['paint', 'layout', 'strict', 'content'].some(value => (css.contain || '').includes(value));
}
function getContainingBlock(element) {
  let currentNode = getParentNode(element);
  while (isHTMLElement(currentNode) && !isLastTraversableNode(currentNode)) {
    if (isContainingBlock(currentNode)) {
      return currentNode;
    } else {
      currentNode = getParentNode(currentNode);
    }
  }
  return null;
}
function isWebKit() {
  if (typeof CSS === 'undefined' || !CSS.supports) return false;
  return CSS.supports('-webkit-backdrop-filter', 'none');
}
function isLastTraversableNode(node) {
  return ['html', 'body', '#document'].includes(getNodeName(node));
}
function getComputedStyle(element) {
  return getWindow(element).getComputedStyle(element);
}
function getNodeScroll(element) {
  if (isElement(element)) {
    return {
      scrollLeft: element.scrollLeft,
      scrollTop: element.scrollTop
    };
  }
  return {
    scrollLeft: element.pageXOffset,
    scrollTop: element.pageYOffset
  };
}
function getParentNode(node) {
  if (getNodeName(node) === 'html') {
    return node;
  }
  const result =
  // Step into the shadow DOM of the parent of a slotted node.
  node.assignedSlot ||
  // DOM Element detected.
  node.parentNode ||
  // ShadowRoot detected.
  isShadowRoot(node) && node.host ||
  // Fallback.
  getDocumentElement(node);
  return isShadowRoot(result) ? result.host : result;
}
function getNearestOverflowAncestor(node) {
  const parentNode = getParentNode(node);
  if (isLastTraversableNode(parentNode)) {
    return node.ownerDocument ? node.ownerDocument.body : node.body;
  }
  if (isHTMLElement(parentNode) && isOverflowElement(parentNode)) {
    return parentNode;
  }
  return getNearestOverflowAncestor(parentNode);
}
function getOverflowAncestors(node, list, traverseIframes) {
  var _node$ownerDocument2;
  if (list === void 0) {
    list = [];
  }
  if (traverseIframes === void 0) {
    traverseIframes = true;
  }
  const scrollableAncestor = getNearestOverflowAncestor(node);
  const isBody = scrollableAncestor === ((_node$ownerDocument2 = node.ownerDocument) == null ? void 0 : _node$ownerDocument2.body);
  const win = getWindow(scrollableAncestor);
  if (isBody) {
    return list.concat(win, win.visualViewport || [], isOverflowElement(scrollableAncestor) ? scrollableAncestor : [], win.frameElement && traverseIframes ? getOverflowAncestors(win.frameElement) : []);
  }
  return list.concat(scrollableAncestor, getOverflowAncestors(scrollableAncestor, [], traverseIframes));
}

function getCssDimensions(element) {
  const css = getComputedStyle(element);
  // In testing environments, the `width` and `height` properties are empty
  // strings for SVG elements, returning NaN. Fallback to `0` in this case.
  let width = parseFloat(css.width) || 0;
  let height = parseFloat(css.height) || 0;
  const hasOffset = isHTMLElement(element);
  const offsetWidth = hasOffset ? element.offsetWidth : width;
  const offsetHeight = hasOffset ? element.offsetHeight : height;
  const shouldFallback = round(width) !== offsetWidth || round(height) !== offsetHeight;
  if (shouldFallback) {
    width = offsetWidth;
    height = offsetHeight;
  }
  return {
    width,
    height,
    $: shouldFallback
  };
}

function unwrapElement(element) {
  return !isElement(element) ? element.contextElement : element;
}

function getScale(element) {
  const domElement = unwrapElement(element);
  if (!isHTMLElement(domElement)) {
    return createCoords(1);
  }
  const rect = domElement.getBoundingClientRect();
  const {
    width,
    height,
    $
  } = getCssDimensions(domElement);
  let x = ($ ? round(rect.width) : rect.width) / width;
  let y = ($ ? round(rect.height) : rect.height) / height;

  // 0, NaN, or Infinity should always fallback to 1.

  if (!x || !Number.isFinite(x)) {
    x = 1;
  }
  if (!y || !Number.isFinite(y)) {
    y = 1;
  }
  return {
    x,
    y
  };
}

const noOffsets = /*#__PURE__*/createCoords(0);
function getVisualOffsets(element) {
  const win = getWindow(element);
  if (!isWebKit() || !win.visualViewport) {
    return noOffsets;
  }
  return {
    x: win.visualViewport.offsetLeft,
    y: win.visualViewport.offsetTop
  };
}
function shouldAddVisualOffsets(element, isFixed, floatingOffsetParent) {
  if (isFixed === void 0) {
    isFixed = false;
  }
  if (!floatingOffsetParent || isFixed && floatingOffsetParent !== getWindow(element)) {
    return false;
  }
  return isFixed;
}

function getBoundingClientRect(element, includeScale, isFixedStrategy, offsetParent) {
  if (includeScale === void 0) {
    includeScale = false;
  }
  if (isFixedStrategy === void 0) {
    isFixedStrategy = false;
  }
  const clientRect = element.getBoundingClientRect();
  const domElement = unwrapElement(element);
  let scale = createCoords(1);
  if (includeScale) {
    if (offsetParent) {
      if (isElement(offsetParent)) {
        scale = getScale(offsetParent);
      }
    } else {
      scale = getScale(element);
    }
  }
  const visualOffsets = shouldAddVisualOffsets(domElement, isFixedStrategy, offsetParent) ? getVisualOffsets(domElement) : createCoords(0);
  let x = (clientRect.left + visualOffsets.x) / scale.x;
  let y = (clientRect.top + visualOffsets.y) / scale.y;
  let width = clientRect.width / scale.x;
  let height = clientRect.height / scale.y;
  if (domElement) {
    const win = getWindow(domElement);
    const offsetWin = offsetParent && isElement(offsetParent) ? getWindow(offsetParent) : offsetParent;
    let currentWin = win;
    let currentIFrame = currentWin.frameElement;
    while (currentIFrame && offsetParent && offsetWin !== currentWin) {
      const iframeScale = getScale(currentIFrame);
      const iframeRect = currentIFrame.getBoundingClientRect();
      const css = getComputedStyle(currentIFrame);
      const left = iframeRect.left + (currentIFrame.clientLeft + parseFloat(css.paddingLeft)) * iframeScale.x;
      const top = iframeRect.top + (currentIFrame.clientTop + parseFloat(css.paddingTop)) * iframeScale.y;
      x *= iframeScale.x;
      y *= iframeScale.y;
      width *= iframeScale.x;
      height *= iframeScale.y;
      x += left;
      y += top;
      currentWin = getWindow(currentIFrame);
      currentIFrame = currentWin.frameElement;
    }
  }
  return rectToClientRect({
    width,
    height,
    x,
    y
  });
}

const topLayerSelectors = [':popover-open', ':modal'];
function isTopLayer(floating) {
  return topLayerSelectors.some(selector => {
    try {
      return floating.matches(selector);
    } catch (e) {
      return false;
    }
  });
}

function convertOffsetParentRelativeRectToViewportRelativeRect(_ref) {
  let {
    elements,
    rect,
    offsetParent,
    strategy
  } = _ref;
  const isFixed = strategy === 'fixed';
  const documentElement = getDocumentElement(offsetParent);
  const topLayer = elements ? isTopLayer(elements.floating) : false;
  if (offsetParent === documentElement || topLayer && isFixed) {
    return rect;
  }
  let scroll = {
    scrollLeft: 0,
    scrollTop: 0
  };
  let scale = createCoords(1);
  const offsets = createCoords(0);
  const isOffsetParentAnElement = isHTMLElement(offsetParent);
  if (isOffsetParentAnElement || !isOffsetParentAnElement && !isFixed) {
    if (getNodeName(offsetParent) !== 'body' || isOverflowElement(documentElement)) {
      scroll = getNodeScroll(offsetParent);
    }
    if (isHTMLElement(offsetParent)) {
      const offsetRect = getBoundingClientRect(offsetParent);
      scale = getScale(offsetParent);
      offsets.x = offsetRect.x + offsetParent.clientLeft;
      offsets.y = offsetRect.y + offsetParent.clientTop;
    }
  }
  return {
    width: rect.width * scale.x,
    height: rect.height * scale.y,
    x: rect.x * scale.x - scroll.scrollLeft * scale.x + offsets.x,
    y: rect.y * scale.y - scroll.scrollTop * scale.y + offsets.y
  };
}

function getClientRects(element) {
  return Array.from(element.getClientRects());
}

function getWindowScrollBarX(element) {
  // If <html> has a CSS width greater than the viewport, then this will be
  // incorrect for RTL.
  return getBoundingClientRect(getDocumentElement(element)).left + getNodeScroll(element).scrollLeft;
}

// Gets the entire size of the scrollable document area, even extending outside
// of the `<html>` and `<body>` rect bounds if horizontally scrollable.
function getDocumentRect(element) {
  const html = getDocumentElement(element);
  const scroll = getNodeScroll(element);
  const body = element.ownerDocument.body;
  const width = max(html.scrollWidth, html.clientWidth, body.scrollWidth, body.clientWidth);
  const height = max(html.scrollHeight, html.clientHeight, body.scrollHeight, body.clientHeight);
  let x = -scroll.scrollLeft + getWindowScrollBarX(element);
  const y = -scroll.scrollTop;
  if (getComputedStyle(body).direction === 'rtl') {
    x += max(html.clientWidth, body.clientWidth) - width;
  }
  return {
    width,
    height,
    x,
    y
  };
}

function getViewportRect(element, strategy) {
  const win = getWindow(element);
  const html = getDocumentElement(element);
  const visualViewport = win.visualViewport;
  let width = html.clientWidth;
  let height = html.clientHeight;
  let x = 0;
  let y = 0;
  if (visualViewport) {
    width = visualViewport.width;
    height = visualViewport.height;
    const visualViewportBased = isWebKit();
    if (!visualViewportBased || visualViewportBased && strategy === 'fixed') {
      x = visualViewport.offsetLeft;
      y = visualViewport.offsetTop;
    }
  }
  return {
    width,
    height,
    x,
    y
  };
}

// Returns the inner client rect, subtracting scrollbars if present.
function getInnerBoundingClientRect(element, strategy) {
  const clientRect = getBoundingClientRect(element, true, strategy === 'fixed');
  const top = clientRect.top + element.clientTop;
  const left = clientRect.left + element.clientLeft;
  const scale = isHTMLElement(element) ? getScale(element) : createCoords(1);
  const width = element.clientWidth * scale.x;
  const height = element.clientHeight * scale.y;
  const x = left * scale.x;
  const y = top * scale.y;
  return {
    width,
    height,
    x,
    y
  };
}
function getClientRectFromClippingAncestor(element, clippingAncestor, strategy) {
  let rect;
  if (clippingAncestor === 'viewport') {
    rect = getViewportRect(element, strategy);
  } else if (clippingAncestor === 'document') {
    rect = getDocumentRect(getDocumentElement(element));
  } else if (isElement(clippingAncestor)) {
    rect = getInnerBoundingClientRect(clippingAncestor, strategy);
  } else {
    const visualOffsets = getVisualOffsets(element);
    rect = {
      ...clippingAncestor,
      x: clippingAncestor.x - visualOffsets.x,
      y: clippingAncestor.y - visualOffsets.y
    };
  }
  return rectToClientRect(rect);
}
function hasFixedPositionAncestor(element, stopNode) {
  const parentNode = getParentNode(element);
  if (parentNode === stopNode || !isElement(parentNode) || isLastTraversableNode(parentNode)) {
    return false;
  }
  return getComputedStyle(parentNode).position === 'fixed' || hasFixedPositionAncestor(parentNode, stopNode);
}

// A "clipping ancestor" is an `overflow` element with the characteristic of
// clipping (or hiding) child elements. This returns all clipping ancestors
// of the given element up the tree.
function getClippingElementAncestors(element, cache) {
  const cachedResult = cache.get(element);
  if (cachedResult) {
    return cachedResult;
  }
  let result = getOverflowAncestors(element, [], false).filter(el => isElement(el) && getNodeName(el) !== 'body');
  let currentContainingBlockComputedStyle = null;
  const elementIsFixed = getComputedStyle(element).position === 'fixed';
  let currentNode = elementIsFixed ? getParentNode(element) : element;

  // https://developer.mozilla.org/en-US/docs/Web/CSS/Containing_block#identifying_the_containing_block
  while (isElement(currentNode) && !isLastTraversableNode(currentNode)) {
    const computedStyle = getComputedStyle(currentNode);
    const currentNodeIsContaining = isContainingBlock(currentNode);
    if (!currentNodeIsContaining && computedStyle.position === 'fixed') {
      currentContainingBlockComputedStyle = null;
    }
    const shouldDropCurrentNode = elementIsFixed ? !currentNodeIsContaining && !currentContainingBlockComputedStyle : !currentNodeIsContaining && computedStyle.position === 'static' && !!currentContainingBlockComputedStyle && ['absolute', 'fixed'].includes(currentContainingBlockComputedStyle.position) || isOverflowElement(currentNode) && !currentNodeIsContaining && hasFixedPositionAncestor(element, currentNode);
    if (shouldDropCurrentNode) {
      // Drop non-containing blocks.
      result = result.filter(ancestor => ancestor !== currentNode);
    } else {
      // Record last containing block for next iteration.
      currentContainingBlockComputedStyle = computedStyle;
    }
    currentNode = getParentNode(currentNode);
  }
  cache.set(element, result);
  return result;
}

// Gets the maximum area that the element is visible in due to any number of
// clipping ancestors.
function getClippingRect(_ref) {
  let {
    element,
    boundary,
    rootBoundary,
    strategy
  } = _ref;
  const elementClippingAncestors = boundary === 'clippingAncestors' ? getClippingElementAncestors(element, this._c) : [].concat(boundary);
  const clippingAncestors = [...elementClippingAncestors, rootBoundary];
  const firstClippingAncestor = clippingAncestors[0];
  const clippingRect = clippingAncestors.reduce((accRect, clippingAncestor) => {
    const rect = getClientRectFromClippingAncestor(element, clippingAncestor, strategy);
    accRect.top = max(rect.top, accRect.top);
    accRect.right = min(rect.right, accRect.right);
    accRect.bottom = min(rect.bottom, accRect.bottom);
    accRect.left = max(rect.left, accRect.left);
    return accRect;
  }, getClientRectFromClippingAncestor(element, firstClippingAncestor, strategy));
  return {
    width: clippingRect.right - clippingRect.left,
    height: clippingRect.bottom - clippingRect.top,
    x: clippingRect.left,
    y: clippingRect.top
  };
}

function getDimensions(element) {
  const {
    width,
    height
  } = getCssDimensions(element);
  return {
    width,
    height
  };
}

function getRectRelativeToOffsetParent(element, offsetParent, strategy) {
  const isOffsetParentAnElement = isHTMLElement(offsetParent);
  const documentElement = getDocumentElement(offsetParent);
  const isFixed = strategy === 'fixed';
  const rect = getBoundingClientRect(element, true, isFixed, offsetParent);
  let scroll = {
    scrollLeft: 0,
    scrollTop: 0
  };
  const offsets = createCoords(0);
  if (isOffsetParentAnElement || !isOffsetParentAnElement && !isFixed) {
    if (getNodeName(offsetParent) !== 'body' || isOverflowElement(documentElement)) {
      scroll = getNodeScroll(offsetParent);
    }
    if (isOffsetParentAnElement) {
      const offsetRect = getBoundingClientRect(offsetParent, true, isFixed, offsetParent);
      offsets.x = offsetRect.x + offsetParent.clientLeft;
      offsets.y = offsetRect.y + offsetParent.clientTop;
    } else if (documentElement) {
      offsets.x = getWindowScrollBarX(documentElement);
    }
  }
  const x = rect.left + scroll.scrollLeft - offsets.x;
  const y = rect.top + scroll.scrollTop - offsets.y;
  return {
    x,
    y,
    width: rect.width,
    height: rect.height
  };
}

function getTrueOffsetParent(element, polyfill) {
  if (!isHTMLElement(element) || getComputedStyle(element).position === 'fixed') {
    return null;
  }
  if (polyfill) {
    return polyfill(element);
  }
  return element.offsetParent;
}

// Gets the closest ancestor positioned element. Handles some edge cases,
// such as table ancestors and cross browser bugs.
function getOffsetParent(element, polyfill) {
  const window = getWindow(element);
  if (!isHTMLElement(element) || isTopLayer(element)) {
    return window;
  }
  let offsetParent = getTrueOffsetParent(element, polyfill);
  while (offsetParent && isTableElement(offsetParent) && getComputedStyle(offsetParent).position === 'static') {
    offsetParent = getTrueOffsetParent(offsetParent, polyfill);
  }
  if (offsetParent && (getNodeName(offsetParent) === 'html' || getNodeName(offsetParent) === 'body' && getComputedStyle(offsetParent).position === 'static' && !isContainingBlock(offsetParent))) {
    return window;
  }
  return offsetParent || getContainingBlock(element) || window;
}

const getElementRects = async function (data) {
  const getOffsetParentFn = this.getOffsetParent || getOffsetParent;
  const getDimensionsFn = this.getDimensions;
  return {
    reference: getRectRelativeToOffsetParent(data.reference, await getOffsetParentFn(data.floating), data.strategy),
    floating: {
      x: 0,
      y: 0,
      ...(await getDimensionsFn(data.floating))
    }
  };
};

function isRTL(element) {
  return getComputedStyle(element).direction === 'rtl';
}

const platform = {
  convertOffsetParentRelativeRectToViewportRelativeRect,
  getDocumentElement,
  getClippingRect,
  getOffsetParent,
  getElementRects,
  getClientRects,
  getDimensions,
  getScale,
  isElement,
  isRTL
};

// https://samthor.au/2021/observing-dom/
function observeMove(element, onMove) {
  let io = null;
  let timeoutId;
  const root = getDocumentElement(element);
  function cleanup() {
    var _io;
    clearTimeout(timeoutId);
    (_io = io) == null || _io.disconnect();
    io = null;
  }
  function refresh(skip, threshold) {
    if (skip === void 0) {
      skip = false;
    }
    if (threshold === void 0) {
      threshold = 1;
    }
    cleanup();
    const {
      left,
      top,
      width,
      height
    } = element.getBoundingClientRect();
    if (!skip) {
      onMove();
    }
    if (!width || !height) {
      return;
    }
    const insetTop = floor(top);
    const insetRight = floor(root.clientWidth - (left + width));
    const insetBottom = floor(root.clientHeight - (top + height));
    const insetLeft = floor(left);
    const rootMargin = -insetTop + "px " + -insetRight + "px " + -insetBottom + "px " + -insetLeft + "px";
    const options = {
      rootMargin,
      threshold: max(0, min(1, threshold)) || 1
    };
    let isFirstUpdate = true;
    function handleObserve(entries) {
      const ratio = entries[0].intersectionRatio;
      if (ratio !== threshold) {
        if (!isFirstUpdate) {
          return refresh();
        }
        if (!ratio) {
          timeoutId = setTimeout(() => {
            refresh(false, 1e-7);
          }, 100);
        } else {
          refresh(false, ratio);
        }
      }
      isFirstUpdate = false;
    }

    // Older browsers don't support a `document` as the root and will throw an
    // error.
    try {
      io = new IntersectionObserver(handleObserve, {
        ...options,
        // Handle <iframe>s
        root: root.ownerDocument
      });
    } catch (e) {
      io = new IntersectionObserver(handleObserve, options);
    }
    io.observe(element);
  }
  refresh(true);
  return cleanup;
}

/**
 * Automatically updates the position of the floating element when necessary.
 * Should only be called when the floating element is mounted on the DOM or
 * visible on the screen.
 * @returns cleanup function that should be invoked when the floating element is
 * removed from the DOM or hidden from the screen.
 * @see https://floating-ui.com/docs/autoUpdate
 */
function autoUpdate(reference, floating, update, options) {
  if (options === void 0) {
    options = {};
  }
  const {
    ancestorScroll = true,
    ancestorResize = true,
    elementResize = typeof ResizeObserver === 'function',
    layoutShift = typeof IntersectionObserver === 'function',
    animationFrame = false
  } = options;
  const referenceEl = unwrapElement(reference);
  const ancestors = ancestorScroll || ancestorResize ? [...(referenceEl ? getOverflowAncestors(referenceEl) : []), ...getOverflowAncestors(floating)] : [];
  ancestors.forEach(ancestor => {
    ancestorScroll && ancestor.addEventListener('scroll', update, {
      passive: true
    });
    ancestorResize && ancestor.addEventListener('resize', update);
  });
  const cleanupIo = referenceEl && layoutShift ? observeMove(referenceEl, update) : null;
  let reobserveFrame = -1;
  let resizeObserver = null;
  if (elementResize) {
    resizeObserver = new ResizeObserver(_ref => {
      let [firstEntry] = _ref;
      if (firstEntry && firstEntry.target === referenceEl && resizeObserver) {
        // Prevent update loops when using the `size` middleware.
        // https://github.com/floating-ui/floating-ui/issues/1740
        resizeObserver.unobserve(floating);
        cancelAnimationFrame(reobserveFrame);
        reobserveFrame = requestAnimationFrame(() => {
          var _resizeObserver;
          (_resizeObserver = resizeObserver) == null || _resizeObserver.observe(floating);
        });
      }
      update();
    });
    if (referenceEl && !animationFrame) {
      resizeObserver.observe(referenceEl);
    }
    resizeObserver.observe(floating);
  }
  let frameId;
  let prevRefRect = animationFrame ? getBoundingClientRect(reference) : null;
  if (animationFrame) {
    frameLoop();
  }
  function frameLoop() {
    const nextRefRect = getBoundingClientRect(reference);
    if (prevRefRect && (nextRefRect.x !== prevRefRect.x || nextRefRect.y !== prevRefRect.y || nextRefRect.width !== prevRefRect.width || nextRefRect.height !== prevRefRect.height)) {
      update();
    }
    prevRefRect = nextRefRect;
    frameId = requestAnimationFrame(frameLoop);
  }
  update();
  return () => {
    var _resizeObserver2;
    ancestors.forEach(ancestor => {
      ancestorScroll && ancestor.removeEventListener('scroll', update);
      ancestorResize && ancestor.removeEventListener('resize', update);
    });
    cleanupIo == null || cleanupIo();
    (_resizeObserver2 = resizeObserver) == null || _resizeObserver2.disconnect();
    resizeObserver = null;
    if (animationFrame) {
      cancelAnimationFrame(frameId);
    }
  };
}

/**
 * Optimizes the visibility of the floating element by flipping the `placement`
 * in order to keep it in view when the preferred placement(s) will overflow the
 * clipping boundary. Alternative to `autoPlacement`.
 * @see https://floating-ui.com/docs/flip
 */
const flip = flip$1;

/**
 * Computes the `x` and `y` coordinates that will place the floating element
 * next to a given reference element.
 */
const computePosition = (reference, floating, options) => {
  // This caches the expensive `getClippingElementAncestors` function so that
  // multiple lifecycle resets re-use the same result. It only lives for a
  // single call. If other functions become expensive, we can add them as well.
  const cache = new Map();
  const mergedOptions = {
    platform,
    ...options
  };
  const platformWithCache = {
    ...mergedOptions.platform,
    _c: cache
  };
  return computePosition$1(reference, floating, {
    ...mergedOptions,
    platform: platformWithCache
  });
};

const createContainer = () => {
  const $find = find$("#" + CONTAINER_ID);
  if (!isNull($find)) return $find;
  const $container = create$("div", {
    id: CONTAINER_ID,
    classList: [PREFIX, CONTAINER_ID],
    role: "presentation",
    innerHTML: (
      /*html*/
      `<div role="presentation" class="${cn("backdrop")}"></div>`
    )
  });
  document.body.appendChild($container);
  return $container;
};
class SnowDatePicker {
  /**
   * @param {Element | string} element
   * @param {Options} [options]
   */
  constructor(element, options = {}) {
    this.$container = createContainer();
    this.$hide = create$("div");
    this.components = [];
    this.unsubscribers = [];
    /** Hide calendar when click outside */
    this.handleClickOutside = (event) => {
      if (!this.isShow()) {
        off(document, "click", this.handleClickOutside);
        return;
      }
      const target = event.target;
      const $btn = find$("." + cn("calendarBtn"), this.$target);
      if ($btn && $btn.contains(target)) return;
      if (this.$datepicker.contains(target)) return;
      this.hide();
    };
    const target = typeof element === "string" ? find$(element) : element;
    if (!(target instanceof Element))
      throw error("constructor element shoul be CSS selector or Element");
    const opts = checkSchema(options);
    this.options = opts;
    this.originTemplate = target.outerHTML;
    const { id, className } = target;
    const $target = create$("div", { id: !id ? void 0 : id, className });
    target.replaceWith($target);
    this.$target = $target;
    this.$input = create$("input");
    const instance = createInstance(opts);
    const eventManager = createEventManager(this, opts);
    this.instance = instance;
    this.eventManager = eventManager;
    this.on = eventManager.on;
    this.off = eventManager.off;
    const classList = [PREFIX, cn("calendar")];
    if (this.options.className) classList.push(this.options.className);
    this.$datepicker = create$("div", { classList });
    this.$hide.appendChild(this.$datepicker);
    this.renderDatepicker();
    this.eventSubcribe();
    const { selectedDate, view, shortcuts } = opts;
    const initDate = selectedDate && isDateLike(selectedDate) ? new Date(selectedDate) : null;
    if (initDate) {
      this.setSelectedDate(initDate);
      this.setViewDate(initDate);
    }
    if (view) this.setCurrentView(view);
    if (shortcuts) {
      const keydownHandler = createShortcutsHandler(this);
      on(document, "keydown", (e) => this.isShow() && keydownHandler(e), true);
    }
    if (!opts.inline && !opts.backdrop) this.calendarPositionUpdate();
    this.setTheme(opts.theme);
    this.setSize(opts.size);
  }
  /** Converter instance for convert date to datepicker's locale format */
  get converter() {
    return this.instance.converter;
  }
  /**
   * Current view of calendar
   * @returns {'years' | 'months' | 'days'}
   */
  get currentView() {
    return this.instance.store.currentView;
  }
  /**
   * Current view date
   * @returns {Date}
   */
  get viewDate() {
    return this.instance.store.viewDate;
  }
  /**
   * Selected date of calendar, if not selected returns `null`
   * @returns {Date | null}
   */
  get selectedDate() {
    return this.instance.store.selectedDate;
  }
  /**
   * Focus date of calendar, if not focused returns `null`
   * @returns {Date | null}
   */
  get focusDate() {
    return this.instance.store.state.focusDate();
  }
  /** Rendering datepicker components */
  renderDatepicker() {
    const { instance, options, eventManager } = this;
    const defaultProps = { dp: this, instance, options, eventManager };
    const $controls = create$("div", { className: cn("controls") });
    const $content = create$("div", { className: cn("content") });
    this.$datepicker.replaceChildren($controls, $content);
    this.$datepicker.tabIndex = -1;
    this.components.push(new Controls($controls, defaultProps));
    this.components.push(new Content($content, defaultProps));
    if (this.options.buttons) {
      const $buttonWrapper = create$("div", { className: cn("buttonWrapper") });
      const buttonOptions = isArray(this.options.buttons) ? this.options.buttons : [this.options.buttons];
      buttonOptions.forEach((buttonOption) => {
        const $btn = create$("button", { className: cn("button") });
        $buttonWrapper.appendChild($btn);
        new Button($btn, { ...defaultProps, buttonOption });
      });
      this.$datepicker.appendChild($buttonWrapper);
    }
    if (this.options.inline) {
      this.$datepicker.classList.add("--inline");
      this.$target.replaceWith(this.$datepicker);
      assignIn(this, { $target: this.$datepicker });
    } else new Target(this.$target, defaultProps);
    if (this.options.backdrop) this.$datepicker.classList.add("--center");
  }
  /** Update calendar position automatically */
  calendarPositionUpdate() {
    const { $target, $datepicker } = this;
    const originMap = {
      top: "50% 100%",
      bottom: "50% 0",
      left: "100% 50%",
      right: "0 50%"
    };
    const updatePosition = () => {
      computePosition($target, $datepicker, {
        placement: this.options.position,
        middleware: [flip()]
      }).then(({ x, y, placement }) => {
        if (!this.options) return;
        const tfOrigin = this.options.animation ? {
          transformOrigin: originMap[placement.split("-")[0]]
        } : {};
        Object.assign($datepicker.style, {
          top: `${y}px`,
          left: `${x}px`,
          ...tfOrigin
        });
      });
    };
    this.unsubscribers.push(autoUpdate($target, $datepicker, updatePosition));
  }
  /** Subscrie events */
  eventSubcribe() {
    const { store } = this.instance;
    this.unsubscribers.push(
      effect(
        (date, prevDate) => {
          this.eventManager.trigger("select", { date, prevDate });
        },
        [store.state.date]
      ),
      effect(
        (view, prevView) => {
          this.eventManager.trigger("changeView", { view, prevView });
        },
        [store.state.currentView]
      ),
      effect(
        (date, prevDate) => {
          this.eventManager.trigger("changeViewDate", { date, prevDate });
        },
        [store.state.viewDate]
      )
    );
  }
  /**
   * Set selected date of datepicker
   * @param {*} value The date to select
   * @example
   *
   * setSelectedDate(new Date(2024, 01, 01, 12)) // Thu Feb 01 2024 12:00:00
   * setSelectedDate('invalid') // null
   */
  setSelectedDate(value) {
    const { minDate, maxDate } = this.options;
    const prevDate = this.selectedDate;
    const nextDate = isDateLike(value) ? new Date(value) : null;
    let date = nextDate;
    if (date) {
      if (date < minDate) date = minDate;
      else if (date > maxDate) date = maxDate;
      date.setHours(12), date.setMinutes(0);
      date.setSeconds(0), date.setMilliseconds(0);
    }
    if (prevDate) {
      prevDate.setHours(12), prevDate.setMinutes(0);
      prevDate.setSeconds(0), prevDate.setMilliseconds(0);
    }
    if (Number(date) === Number(prevDate)) return;
    const eventProps = { prevDate, date };
    const beforeResults = this.eventManager.trigger("beforeSelect", eventProps);
    const isCancel = isArray(beforeResults) && beforeResults.some((item) => item === false);
    if (isCancel) return;
    this.instance.store.setSelectedDate(date);
  }
  /**
   * Set view date of datepicker
   * @param {DateLike} value The view date to change.
   */
  setViewDate(value) {
    if (!isDateLike(value)) return;
    const { minDate, maxDate } = this.options;
    let viewDate = new Date(value);
    if (viewDate < minDate) viewDate = minDate;
    else if (viewDate > maxDate) viewDate = maxDate;
    this.instance.store.setViewDate(viewDate);
  }
  /**
   * Set current view of datepicker
   * @param {View} value The view to change. (e.g. `days` | `months` | `years`)
   */
  setCurrentView(value) {
    this.instance.store.setCurrentView(value);
  }
  /**
   * Set focus date of datepicker
   * @param {*} value The value to change
   */
  setFocusDate(value) {
    const date = !isDateLike(value) ? null : new Date(value);
    this.instance.store.state.focusDate(date);
  }
  /**
   * Check arguments date is same date in given view.
   * @param {Date} date The value to check
   * @param {Date} compare The value to compare
   * @param {'days' | 'months' | 'years'} view The view state to use for comparison.
   * @returns {boolean} Returns the value and compare in the same view
   * @example
   *
   * isSameDate(new Date('2024-01-01'), new Date('2024-01-30')) // false
   * isSameDate(new Date('2024-01-01'), new Date('2024-01-30'), 'months') // true
   */
  isSameDate(date, compare, view = "days") {
    const arr1 = parseDate(date).slice(0, 4 - VIEW_ORDER[view]);
    const arr2 = parseDate(compare);
    return arr1.every((value, index) => arr2[index] === value);
  }
  addViewDate(direction) {
    const adder = direction === "prev" ? -1 : 1;
    const { currentView, viewDate } = this;
    const params = parseDate(viewDate).slice(0, 2);
    let [year, month] = params;
    month += 1;
    switch (currentView) {
      case "days":
        month += adder;
        if (month === 0) month = 12, year -= 1;
        else if (month === 13) month = 1, year += 1;
        break;
      case "months":
        year += adder;
        break;
      case "years":
        year += adder * 10;
        break;
    }
    const y = String(year).padStart(4, "0");
    const m = String(month).padStart(2, "0");
    const nextViewDate = /* @__PURE__ */ new Date(`${y}-${m}-01`);
    if (this.options.navigationLoop) {
      const { minDate, maxDate } = this.options;
      if (nextViewDate < minDate) return this.setViewDate(maxDate);
      else if (maxDate < nextViewDate) return this.setViewDate(minDate);
    }
    this.setViewDate(nextViewDate);
  }
  /** Move to next `month` | `years` | `decade` */
  next() {
    this.addViewDate("next");
  }
  /** Move to previous `month` | `years` | `decade` */
  prev() {
    this.addViewDate("prev");
  }
  /** Hide calendar */
  hide() {
    if (this.options.inline || !this.isShow()) return;
    if (this.options.animation) {
      const animationName = this.options.backdrop ? "--fadeOut" : "--scaleDown";
      this.$datepicker.classList.add(animationName);
      const onAnimeEnd = (event) => {
        event.currentTarget.classList.remove(animationName);
        this.$hide.replaceChildren(this.$datepicker);
        off(this.$datepicker, "animationend", onAnimeEnd);
      };
      on(this.$datepicker, "animationend", onAnimeEnd);
    } else {
      this.$hide.replaceChildren(this.$datepicker);
    }
    this.$datepicker.classList.remove("--show");
    this.eventManager.trigger("hide", {});
    if (this.$input !== document.activeElement)
      find$("." + cn("inputBox")).classList.remove("--active");
  }
  /** Show calendar */
  show() {
    if (this.options.inline || this.isShow()) return;
    const $conatiner = this.$container;
    $conatiner.appendChild(this.$datepicker);
    this.$datepicker.classList.add("--show");
    if (this.options.animation) {
      const animationName = this.options.backdrop ? "--fadeIn" : "--scaleUp";
      this.$datepicker.classList.add(animationName);
      const onAnimeEnd = (event) => {
        event.currentTarget.classList.remove(animationName);
        this.$datepicker.focus();
        off(this.$datepicker, "animationend", onAnimeEnd);
      };
      on(this.$datepicker, "animationend", onAnimeEnd);
    } else this.$datepicker.focus();
    find$("." + cn("inputBox")).classList.add("--active");
    on(document, "click", this.handleClickOutside, { capture: true });
    this.eventManager.trigger("show", {});
  }
  /**
   * Set datepicker theme mode
   * @param {'light' | 'dark'} theme
   */
  setTheme(theme) {
    const isDark = theme === "dark";
    if (theme !== "light" && !isDark) return;
    [this.$container, this.$datepicker, this.$target].forEach(
      ($el) => $el.classList[isDark ? "add" : "remove"]("--dark")
    );
  }
  setSize(size) {
    const isSmall = size === "small";
    if (size !== "normal" && !isSmall) return;
    this.$target.classList[isSmall ? "add" : "remove"]("--small");
  }
  /** Datepicker calendar is visible */
  isShow() {
    return !this.$hide.contains(this.$datepicker);
  }
  /** Delete datepicker */
  destroy() {
    this.unsubscribers.forEach((fn) => fn());
    this.$input.outerHTML = this.originTemplate;
    this.components.forEach((component) => component.destroy());
    const elementKeys = ["$datepicker", "$container", "$hide"];
    elementKeys.forEach((key) => get(this, key).remove());
    this.$target.outerHTML = this.originTemplate;
    keysIn(this).forEach((key) => get(this, key) && delete this[key]);
  }
}
/** ID of datepicker elements container */
SnowDatePicker.containerId = CONTAINER_ID;
SnowDatePicker.createContainer = createContainer;

export { SnowDatePicker as default };
